@@ -14,17 +14,13 @@
  * limitations under the License.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <php.h>
 #include <Zend/zend_interfaces.h>
 #include <ext/standard/php_var.h>
-#if PHP_VERSION_ID >= 70000
 #include <zend_smart_str.h>
-#else
-#include <ext/standard/php_smart_str.h>
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
 #endif
 
 #include "phongo_compat.h"
@@ -38,7 +34,7 @@ static PHP_METHOD(MaxKey, __set_state)
 {
 	zval* array;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a", &array) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &array) == FAILURE) {
 		RETURN_FALSE;
 	}
 
@@ -61,7 +57,7 @@ static PHP_METHOD(MaxKey, jsonSerialize)
 */
 static PHP_METHOD(MaxKey, serialize)
 {
-	PHONGO_RETURN_STRING("");
+	RETURN_STRING("");
 } /* }}} */
 
 /* {{{ proto void MongoDB\BSON\MaxKey::unserialize(string $serialized)
@@ -70,15 +66,15 @@ static PHP_METHOD(MaxKey, unserialize)
 {
 	zend_error_handling error_handling;
 	char*               serialized;
-	phongo_zpp_char_len serialized_len;
+	size_t              serialized_len;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling TSRMLS_CC);
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &serialized, &serialized_len) == FAILURE) {
-		zend_restore_error_handling(&error_handling TSRMLS_CC);
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
-	zend_restore_error_handling(&error_handling TSRMLS_CC);
+	zend_restore_error_handling(&error_handling);
 } /* }}} */
 
 /* {{{ MongoDB\BSON\MaxKey function entries */
@@ -107,38 +103,24 @@ static zend_function_entry php_phongo_maxkey_me[] = {
 /* {{{ MongoDB\BSON\MaxKey object handlers */
 static zend_object_handlers php_phongo_handler_maxkey;
 
-static void php_phongo_maxkey_free_object(phongo_free_object_arg* object TSRMLS_DC) /* {{{ */
+static void php_phongo_maxkey_free_object(zend_object* object) /* {{{ */
 {
 	php_phongo_maxkey_t* intern = Z_OBJ_MAXKEY(object);
 
-	zend_object_std_dtor(&intern->std TSRMLS_CC);
-
-#if PHP_VERSION_ID < 70000
-	efree(intern);
-#endif
+	zend_object_std_dtor(&intern->std);
 } /* }}} */
 
-static phongo_create_object_retval php_phongo_maxkey_create_object(zend_class_entry* class_type TSRMLS_DC) /* {{{ */
+static zend_object* php_phongo_maxkey_create_object(zend_class_entry* class_type) /* {{{ */
 {
 	php_phongo_maxkey_t* intern = NULL;
 
 	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_maxkey_t, class_type);
-	zend_object_std_init(&intern->std, class_type TSRMLS_CC);
+	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
 
-#if PHP_VERSION_ID >= 70000
 	intern->std.handlers = &php_phongo_handler_maxkey;
 
 	return &intern->std;
-#else
-	{
-		zend_object_value retval;
-		retval.handle   = zend_objects_store_put(intern, (zend_objects_store_dtor_t) zend_objects_destroy_object, php_phongo_maxkey_free_object, NULL TSRMLS_CC);
-		retval.handlers = &php_phongo_handler_maxkey;
-
-		return retval;
-	}
-#endif
 } /* }}} */
 /* }}} */
 
@@ -147,20 +129,20 @@ void php_phongo_maxkey_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	zend_class_entry ce;
 
 	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\BSON", "MaxKey", php_phongo_maxkey_me);
-	php_phongo_maxkey_ce                = zend_register_internal_class(&ce TSRMLS_CC);
+	php_phongo_maxkey_ce                = zend_register_internal_class(&ce);
 	php_phongo_maxkey_ce->create_object = php_phongo_maxkey_create_object;
 	PHONGO_CE_FINAL(php_phongo_maxkey_ce);
 
-	zend_class_implements(php_phongo_maxkey_ce TSRMLS_CC, 1, php_phongo_maxkey_interface_ce);
-	zend_class_implements(php_phongo_maxkey_ce TSRMLS_CC, 1, php_phongo_json_serializable_ce);
-	zend_class_implements(php_phongo_maxkey_ce TSRMLS_CC, 1, php_phongo_type_ce);
-	zend_class_implements(php_phongo_maxkey_ce TSRMLS_CC, 1, zend_ce_serializable);
+	zend_class_implements(php_phongo_maxkey_ce, 1, php_phongo_maxkey_interface_ce);
+	zend_class_implements(php_phongo_maxkey_ce, 1, php_phongo_json_serializable_ce);
+	zend_class_implements(php_phongo_maxkey_ce, 1, php_phongo_type_ce);
+	zend_class_implements(php_phongo_maxkey_ce, 1, zend_ce_serializable);
 
 	memcpy(&php_phongo_handler_maxkey, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
-#if PHP_VERSION_ID >= 70000
-	php_phongo_handler_maxkey.free_obj = php_phongo_maxkey_free_object;
-	php_phongo_handler_maxkey.offset   = XtOffsetOf(php_phongo_maxkey_t, std);
-#endif
+	/* Re-assign default handler previously removed in php_phongo.c */
+	php_phongo_handler_maxkey.clone_obj = zend_objects_clone_obj;
+	php_phongo_handler_maxkey.free_obj  = php_phongo_maxkey_free_object;
+	php_phongo_handler_maxkey.offset    = XtOffsetOf(php_phongo_maxkey_t, std);
 } /* }}} */
 
 /*
