@@ -14,22 +14,21 @@
  * limitations under the License.
  */
 
+#include <php.h>
+#include <Zend/zend_interfaces.h>
+#include <ext/standard/php_var.h>
+#include <zend_smart_str.h>
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <php.h>
-#include <Zend/zend_interfaces.h>
-#include <ext/standard/php_var.h>
-#if PHP_VERSION_ID >= 70000
-#include <zend_smart_str.h>
-#else
-#include <ext/standard/php_smart_str.h>
-#endif
-
 #include "phongo_compat.h"
 #include "php_phongo.h"
 
+#define PHONGO_OID_SIZE sizeof(((php_phongo_objectid_t*) 0)->oid)
+#define PHONGO_OID_LEN (PHONGO_OID_SIZE - 1)
+
 zend_class_entry* php_phongo_objectid_ce;
 
 /* Initialize the object with a generated value and return whether it was
@@ -48,7 +47,7 @@ static bool php_phongo_objectid_init(php_phongo_objectid_t* intern)
 
 /* Initialize the object from a hex string and return whether it was successful.
  * An exception will be thrown on error. */
-static bool php_phongo_objectid_init_from_hex_string(php_phongo_objectid_t* intern, const char* hex, phongo_zpp_char_len hex_len TSRMLS_DC) /* {{{ */
+static bool php_phongo_objectid_init_from_hex_string(php_phongo_objectid_t* intern, const char* hex, size_t hex_len) /* {{{ */
 {
 	if (bson_oid_is_valid(hex, hex_len)) {
 		bson_oid_t oid;
@@ -60,32 +59,24 @@ static bool php_phongo_objectid_init_from_hex_string(php_phongo_objectid_t* inte
 		return true;
 	}
 
-	phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Error parsing ObjectId string: %s", hex);
+	phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Error parsing ObjectId string: %s", hex);
 
 	return false;
 } /* }}} */
 
 /* Initialize the object from a HashTable and return whether it was successful.
  * An exception will be thrown on error. */
-static bool php_phongo_objectid_init_from_hash(php_phongo_objectid_t* intern, HashTable* props TSRMLS_DC) /* {{{ */
+static bool php_phongo_objectid_init_from_hash(php_phongo_objectid_t* intern, HashTable* props) /* {{{ */
 {
-#if PHP_VERSION_ID >= 70000
 	zval* z_oid;
 
 	z_oid = zend_hash_str_find(props, "oid", sizeof("oid") - 1);
 
 	if (z_oid && Z_TYPE_P(z_oid) == IS_STRING) {
-		return php_phongo_objectid_init_from_hex_string(intern, Z_STRVAL_P(z_oid), Z_STRLEN_P(z_oid) TSRMLS_CC);
+		return php_phongo_objectid_init_from_hex_string(intern, Z_STRVAL_P(z_oid), Z_STRLEN_P(z_oid));
 	}
-#else
-	zval** z_oid;
 
-	if (zend_hash_find(props, "oid", sizeof("oid"), (void**) &z_oid) == SUCCESS && Z_TYPE_PP(z_oid) == IS_STRING) {
-		return php_phongo_objectid_init_from_hex_string(intern, Z_STRVAL_PP(z_oid), Z_STRLEN_PP(z_oid) TSRMLS_CC);
-	}
-#endif
-
-	phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "%s initialization requires \"oid\" string field", ZSTR_VAL(php_phongo_objectid_ce->name));
+	phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "%s initialization requires \"oid\" string field", ZSTR_VAL(php_phongo_objectid_ce->name));
 	return false;
 } /* }}} */
 
@@ -96,19 +87,19 @@ static PHP_METHOD(ObjectId, __construct)
 	php_phongo_objectid_t* intern;
 	zend_error_handling    error_handling;
 	char*                  id = NULL;
-	phongo_zpp_char_len    id_len;
+	size_t                 id_len;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling TSRMLS_CC);
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	intern = Z_OBJECTID_OBJ_P(getThis());
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s!", &id, &id_len) == FAILURE) {
-		zend_restore_error_handling(&error_handling TSRMLS_CC);
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|s!", &id, &id_len) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
-	zend_restore_error_handling(&error_handling TSRMLS_CC);
+	zend_restore_error_handling(&error_handling);
 
 	if (id) {
-		php_phongo_objectid_init_from_hex_string(intern, id, id_len TSRMLS_CC);
+		php_phongo_objectid_init_from_hex_string(intern, id, id_len);
 	} else {
 		php_phongo_objectid_init(intern);
 	}
@@ -139,7 +130,7 @@ static PHP_METHOD(ObjectId, __set_state)
 	HashTable*             props;
 	zval*                  array;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a", &array) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &array) == FAILURE) {
 		RETURN_FALSE;
 	}
 
@@ -148,7 +139,7 @@ static PHP_METHOD(ObjectId, __set_state)
 	intern = Z_OBJECTID_OBJ_P(return_value);
 	props  = Z_ARRVAL_P(array);
 
-	php_phongo_objectid_init_from_hash(intern, props TSRMLS_CC);
+	php_phongo_objectid_init_from_hash(intern, props);
 } /* }}} */
 
 /* {{{ proto string MongoDB\BSON\ObjectId::__toString()
@@ -163,7 +154,7 @@ static PHP_METHOD(ObjectId, __toString)
 		return;
 	}
 
-	PHONGO_RETURN_STRINGL(intern->oid, 24);
+	RETURN_STRINGL(intern->oid, PHONGO_OID_LEN);
 } /* }}} */
 
 /* {{{ proto array MongoDB\BSON\ObjectId::jsonSerialize()
@@ -179,7 +170,7 @@ static PHP_METHOD(ObjectId, jsonSerialize)
 	intern = Z_OBJECTID_OBJ_P(getThis());
 
 	array_init_size(return_value, 1);
-	ADD_ASSOC_STRINGL(return_value, "$oid", intern->oid, 24);
+	ADD_ASSOC_STRINGL(return_value, "$oid", intern->oid, PHONGO_OID_LEN);
 } /* }}} */
 
 /* {{{ proto string MongoDB\BSON\ObjectId::serialize()
@@ -187,7 +178,7 @@ static PHP_METHOD(ObjectId, jsonSerialize)
 static PHP_METHOD(ObjectId, serialize)
 {
 	php_phongo_objectid_t* intern;
-	ZVAL_RETVAL_TYPE       retval;
+	zval                   retval;
 	php_serialize_data_t   var_hash;
 	smart_str              buf = { 0 };
 
@@ -197,17 +188,11 @@ static PHP_METHOD(ObjectId, serialize)
 		return;
 	}
 
-#if PHP_VERSION_ID >= 70000
-	array_init_size(&retval, 2);
-	ADD_ASSOC_STRINGL(&retval, "oid", intern->oid, 24);
-#else
-	ALLOC_INIT_ZVAL(retval);
-	array_init_size(retval, 2);
-	ADD_ASSOC_STRINGL(retval, "oid", intern->oid, 24);
-#endif
+	array_init_size(&retval, 1);
+	ADD_ASSOC_STRINGL(&retval, "oid", intern->oid, PHONGO_OID_LEN);
 
 	PHP_VAR_SERIALIZE_INIT(var_hash);
-	php_var_serialize(&buf, &retval, &var_hash TSRMLS_CC);
+	php_var_serialize(&buf, &retval, &var_hash);
 	smart_str_0(&buf);
 	PHP_VAR_SERIALIZE_DESTROY(var_hash);
 
@@ -224,42 +209,31 @@ static PHP_METHOD(ObjectId, unserialize)
 	php_phongo_objectid_t* intern;
 	zend_error_handling    error_handling;
 	char*                  serialized;
-	phongo_zpp_char_len    serialized_len;
-#if PHP_VERSION_ID >= 70000
-	zval props;
-#else
-	zval* props;
-#endif
+	size_t                 serialized_len;
+	zval                   props;
 	php_unserialize_data_t var_hash;
 
 	intern = Z_OBJECTID_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling TSRMLS_CC);
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &serialized, &serialized_len) == FAILURE) {
-		zend_restore_error_handling(&error_handling TSRMLS_CC);
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
-	zend_restore_error_handling(&error_handling TSRMLS_CC);
+	zend_restore_error_handling(&error_handling);
 
-#if PHP_VERSION_ID < 70000
-	ALLOC_INIT_ZVAL(props);
-#endif
 	PHP_VAR_UNSERIALIZE_INIT(var_hash);
-	if (!php_var_unserialize(&props, (const unsigned char**) &serialized, (unsigned char*) serialized + serialized_len, &var_hash TSRMLS_CC)) {
+	if (!php_var_unserialize(&props, (const unsigned char**) &serialized, (unsigned char*) serialized + serialized_len, &var_hash)) {
 		zval_ptr_dtor(&props);
-		phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "%s unserialization failed", ZSTR_VAL(php_phongo_objectid_ce->name));
+		phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "%s unserialization failed", ZSTR_VAL(php_phongo_objectid_ce->name));
 
 		PHP_VAR_UNSERIALIZE_DESTROY(var_hash);
 		return;
 	}
 	PHP_VAR_UNSERIALIZE_DESTROY(var_hash);
 
-#if PHP_VERSION_ID >= 70000
-	php_phongo_objectid_init_from_hash(intern, HASH_OF(&props) TSRMLS_CC);
-#else
-	php_phongo_objectid_init_from_hash(intern, HASH_OF(props) TSRMLS_CC);
-#endif
+	php_phongo_objectid_init_from_hash(intern, HASH_OF(&props));
 	zval_ptr_dtor(&props);
 } /* }}} */
 
@@ -296,47 +270,52 @@ static zend_function_entry php_phongo_objectid_me[] = {
 /* {{{ MongoDB\BSON\ObjectId object handlers */
 static zend_object_handlers php_phongo_handler_objectid;
 
-static void php_phongo_objectid_free_object(phongo_free_object_arg* object TSRMLS_DC) /* {{{ */
+static void php_phongo_objectid_free_object(zend_object* object) /* {{{ */
 {
 	php_phongo_objectid_t* intern = Z_OBJ_OBJECTID(object);
 
-	zend_object_std_dtor(&intern->std TSRMLS_CC);
+	zend_object_std_dtor(&intern->std);
 
 	if (intern->properties) {
 		zend_hash_destroy(intern->properties);
 		FREE_HASHTABLE(intern->properties);
 	}
-
-#if PHP_VERSION_ID < 70000
-	efree(intern);
-#endif
 } /* }}} */
 
-static phongo_create_object_retval php_phongo_objectid_create_object(zend_class_entry* class_type TSRMLS_DC) /* {{{ */
+static zend_object* php_phongo_objectid_create_object(zend_class_entry* class_type) /* {{{ */
 {
 	php_phongo_objectid_t* intern = NULL;
 
 	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_objectid_t, class_type);
 
-	zend_object_std_init(&intern->std, class_type TSRMLS_CC);
+	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
 
-#if PHP_VERSION_ID >= 70000
 	intern->std.handlers = &php_phongo_handler_objectid;
 
 	return &intern->std;
-#else
-	{
-		zend_object_value retval;
-		retval.handle   = zend_objects_store_put(intern, (zend_objects_store_dtor_t) zend_objects_destroy_object, php_phongo_objectid_free_object, NULL TSRMLS_CC);
-		retval.handlers = &php_phongo_handler_objectid;
-
-		return retval;
-	}
-#endif
 } /* }}} */
 
-static int php_phongo_objectid_compare_objects(zval* o1, zval* o2 TSRMLS_DC) /* {{{ */
+static zend_object* php_phongo_objectid_clone_object(zval* object) /* {{{ */
+{
+	php_phongo_objectid_t* intern;
+	php_phongo_objectid_t* new_intern;
+	zend_object*           new_object;
+
+	intern     = Z_OBJECTID_OBJ_P(object);
+	new_object = php_phongo_objectid_create_object(Z_OBJCE_P(object));
+
+	new_intern = Z_OBJ_OBJECTID(new_object);
+	zend_objects_clone_members(&new_intern->std, &intern->std);
+
+	// Use memcpy to copy bson value to avoid converting to string and back
+	memcpy(&new_intern->oid, &intern->oid, PHONGO_OID_SIZE);
+	new_intern->initialized = true;
+
+	return new_object;
+} /* }}} */
+
+static int php_phongo_objectid_compare_objects(zval* o1, zval* o2) /* {{{ */
 {
 	php_phongo_objectid_t* intern1;
 	php_phongo_objectid_t* intern2;
@@ -347,15 +326,7 @@ static int php_phongo_objectid_compare_objects(zval* o1, zval* o2 TSRMLS_DC) /*
 	return strcmp(intern1->oid, intern2->oid);
 } /* }}} */
 
-static HashTable* php_phongo_objectid_get_gc(zval* object, phongo_get_gc_table table, int* n TSRMLS_DC) /* {{{ */
-{
-	*table = NULL;
-	*n     = 0;
-
-	return Z_OBJECTID_OBJ_P(object)->properties;
-} /* }}} */
-
-static HashTable* php_phongo_objectid_get_properties_hash(zval* object, bool is_debug TSRMLS_DC) /* {{{ */
+static HashTable* php_phongo_objectid_get_properties_hash(zval* object, bool is_debug) /* {{{ */
 {
 	php_phongo_objectid_t* intern;
 	HashTable*             props;
@@ -368,35 +339,25 @@ static HashTable* php_phongo_objectid_get_properties_hash(zval* object, bool is_
 		return props;
 	}
 
-#if PHP_VERSION_ID >= 70000
 	{
 		zval zv;
 
 		ZVAL_STRING(&zv, intern->oid);
 		zend_hash_str_update(props, "oid", sizeof("oid") - 1, &zv);
 	}
-#else
-	{
-		zval* zv;
-
-		MAKE_STD_ZVAL(zv);
-		ZVAL_STRING(zv, intern->oid, 1);
-		zend_hash_update(props, "oid", sizeof("oid"), &zv, sizeof(zv), NULL);
-	}
-#endif
 
 	return props;
 } /* }}} */
 
-static HashTable* php_phongo_objectid_get_debug_info(zval* object, int* is_temp TSRMLS_DC) /* {{{ */
+static HashTable* php_phongo_objectid_get_debug_info(zval* object, int* is_temp) /* {{{ */
 {
 	*is_temp = 1;
-	return php_phongo_objectid_get_properties_hash(object, true TSRMLS_CC);
+	return php_phongo_objectid_get_properties_hash(object, true);
 } /* }}} */
 
-static HashTable* php_phongo_objectid_get_properties(zval* object TSRMLS_DC) /* {{{ */
+static HashTable* php_phongo_objectid_get_properties(zval* object) /* {{{ */
 {
-	return php_phongo_objectid_get_properties_hash(object, false TSRMLS_CC);
+	return php_phongo_objectid_get_properties_hash(object, false);
 } /* }}} */
 /* }}} */
 
@@ -405,24 +366,22 @@ void php_phongo_objectid_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	zend_class_entry ce;
 
 	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\BSON", "ObjectId", php_phongo_objectid_me);
-	php_phongo_objectid_ce                = zend_register_internal_class(&ce TSRMLS_CC);
+	php_phongo_objectid_ce                = zend_register_internal_class(&ce);
 	php_phongo_objectid_ce->create_object = php_phongo_objectid_create_object;
 	PHONGO_CE_FINAL(php_phongo_objectid_ce);
 
-	zend_class_implements(php_phongo_objectid_ce TSRMLS_CC, 1, php_phongo_objectid_interface_ce);
-	zend_class_implements(php_phongo_objectid_ce TSRMLS_CC, 1, php_phongo_json_serializable_ce);
-	zend_class_implements(php_phongo_objectid_ce TSRMLS_CC, 1, php_phongo_type_ce);
-	zend_class_implements(php_phongo_objectid_ce TSRMLS_CC, 1, zend_ce_serializable);
+	zend_class_implements(php_phongo_objectid_ce, 1, php_phongo_objectid_interface_ce);
+	zend_class_implements(php_phongo_objectid_ce, 1, php_phongo_json_serializable_ce);
+	zend_class_implements(php_phongo_objectid_ce, 1, php_phongo_type_ce);
+	zend_class_implements(php_phongo_objectid_ce, 1, zend_ce_serializable);
 
 	memcpy(&php_phongo_handler_objectid, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
+	php_phongo_handler_objectid.clone_obj       = php_phongo_objectid_clone_object;
 	php_phongo_handler_objectid.compare_objects = php_phongo_objectid_compare_objects;
 	php_phongo_handler_objectid.get_debug_info  = php_phongo_objectid_get_debug_info;
-	php_phongo_handler_objectid.get_gc          = php_phongo_objectid_get_gc;
 	php_phongo_handler_objectid.get_properties  = php_phongo_objectid_get_properties;
-#if PHP_VERSION_ID >= 70000
-	php_phongo_handler_objectid.free_obj = php_phongo_objectid_free_object;
-	php_phongo_handler_objectid.offset   = XtOffsetOf(php_phongo_objectid_t, std);
-#endif
+	php_phongo_handler_objectid.free_obj        = php_phongo_objectid_free_object;
+	php_phongo_handler_objectid.offset          = XtOffsetOf(php_phongo_objectid_t, std);
 } /* }}} */
 
 /*
