@@ -14,58 +14,42 @@
  * limitations under the License.
  */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
-
 #include <math.h>
 
 #include <php.h>
 #include <Zend/zend_interfaces.h>
 #include <ext/date/php_date.h>
 #include <ext/standard/php_var.h>
-#if PHP_VERSION_ID >= 70000
-# include <zend_smart_str.h>
-#else
-# include <ext/standard/php_smart_str.h>
-#endif
-#ifdef PHP_WIN32
-# include "win32/time.h"
+#include <zend_smart_str.h>
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
 #endif
 
+#ifdef PHP_WIN32
+#include "win32/time.h"
+#endif
 #include "phongo_compat.h"
 #include "php_phongo.h"
 
-zend_class_entry *php_phongo_utcdatetime_ce;
+zend_class_entry* php_phongo_utcdatetime_ce;
 
 /* Initialize the object and return whether it was successful. */
-static bool php_phongo_utcdatetime_init(php_phongo_utcdatetime_t *intern, int64_t milliseconds) /* {{{ */
+static bool php_phongo_utcdatetime_init(php_phongo_utcdatetime_t* intern, int64_t milliseconds) /* {{{ */
 {
 	intern->milliseconds = milliseconds;
-	intern->initialized = true;
+	intern->initialized  = true;
 
 	return true;
 } /* }}} */
 
 /* Initialize the object from a numeric string and return whether it was
  * successful. An exception will be thrown on error. */
-static bool php_phongo_utcdatetime_init_from_string(php_phongo_utcdatetime_t *intern, const char *s_milliseconds, phongo_zpp_char_len s_milliseconds_len TSRMLS_DC) /* {{{ */
+static bool php_phongo_utcdatetime_init_from_string(php_phongo_utcdatetime_t* intern, const char* s_milliseconds, size_t s_milliseconds_len TSRMLS_DC) /* {{{ */
 {
 	int64_t milliseconds;
-	char *endptr = NULL;
 
-	errno = 0;
-
-	milliseconds = bson_ascii_strtoll(s_milliseconds, &endptr, 10);
-
-	/* errno will set errno if conversion fails; however, we do not need to
-	 * specify the type of error.
-	 *
-	 * Note: bson_ascii_strtoll() does not properly detect out-of-range values
-	 * (see: CDRIVER-1377). strtoll() would be preferable, but it is not
-	 * available on all platforms (e.g. HP-UX), and atoll() provides no error
-	 * reporting at all. */
-	if (errno || (endptr && endptr != ((const char *)s_milliseconds + s_milliseconds_len))) {
+	if (!php_phongo_parse_int64(&milliseconds, s_milliseconds, s_milliseconds_len)) {
 		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Error parsing \"%s\" as 64-bit integer for %s initialization", s_milliseconds, ZSTR_VAL(php_phongo_utcdatetime_ce->name));
 		return false;
 	}
@@ -75,29 +59,17 @@ static bool php_phongo_utcdatetime_init_from_string(php_phongo_utcdatetime_t *in
 
 /* Initialize the object from a HashTable and return whether it was successful.
  * An exception will be thrown on error. */
-static bool php_phongo_utcdatetime_init_from_hash(php_phongo_utcdatetime_t *intern, HashTable *props TSRMLS_DC) /* {{{ */
+static bool php_phongo_utcdatetime_init_from_hash(php_phongo_utcdatetime_t* intern, HashTable* props TSRMLS_DC) /* {{{ */
 {
-#if PHP_VERSION_ID >= 70000
-	zval *milliseconds;
+	zval* milliseconds;
 
-	if ((milliseconds = zend_hash_str_find(props, "milliseconds", sizeof("milliseconds")-1)) && Z_TYPE_P(milliseconds) == IS_LONG) {
+	if ((milliseconds = zend_hash_str_find(props, "milliseconds", sizeof("milliseconds") - 1)) && Z_TYPE_P(milliseconds) == IS_LONG) {
 		return php_phongo_utcdatetime_init(intern, Z_LVAL_P(milliseconds));
 	}
 
-	if ((milliseconds = zend_hash_str_find(props, "milliseconds", sizeof("milliseconds")-1)) && Z_TYPE_P(milliseconds) == IS_STRING) {
+	if ((milliseconds = zend_hash_str_find(props, "milliseconds", sizeof("milliseconds") - 1)) && Z_TYPE_P(milliseconds) == IS_STRING) {
 		return php_phongo_utcdatetime_init_from_string(intern, Z_STRVAL_P(milliseconds), Z_STRLEN_P(milliseconds) TSRMLS_CC);
 	}
-#else
-	zval **milliseconds;
-
-	if (zend_hash_find(props, "milliseconds", sizeof("milliseconds"), (void**) &milliseconds) == SUCCESS && Z_TYPE_PP(milliseconds) == IS_LONG) {
-		return php_phongo_utcdatetime_init(intern, Z_LVAL_PP(milliseconds));
-	}
-
-	if (zend_hash_find(props, "milliseconds", sizeof("milliseconds"), (void**) &milliseconds) == SUCCESS && Z_TYPE_PP(milliseconds) == IS_STRING) {
-		return php_phongo_utcdatetime_init_from_string(intern, Z_STRVAL_PP(milliseconds), Z_STRLEN_PP(milliseconds) TSRMLS_CC);
-	}
-#endif
 
 	phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "%s initialization requires \"milliseconds\" integer or numeric string field", ZSTR_VAL(php_phongo_utcdatetime_ce->name));
 	return false;
@@ -105,24 +77,24 @@ static bool php_phongo_utcdatetime_init_from_hash(php_phongo_utcdatetime_t *inte
 
 /* Initialize the object from the current time and return whether it was
  * successful. */
-static bool php_phongo_utcdatetime_init_from_current_time(php_phongo_utcdatetime_t *intern) /* {{{ */
+static bool php_phongo_utcdatetime_init_from_current_time(php_phongo_utcdatetime_t* intern) /* {{{ */
 {
 	int64_t        sec, usec;
 	struct timeval cur_time;
 
 	gettimeofday(&cur_time, NULL);
-	sec = cur_time.tv_sec;
+	sec  = cur_time.tv_sec;
 	usec = cur_time.tv_usec;
 
 	intern->milliseconds = (sec * 1000) + (usec / 1000);
-	intern->initialized = true;
+	intern->initialized  = true;
 
 	return true;
 } /* }}} */
 
 /* Initialize the object from a DateTime object and return whether it was
  * successful. */
-static bool php_phongo_utcdatetime_init_from_date(php_phongo_utcdatetime_t *intern, php_date_obj *datetime_obj) /* {{{ */
+static bool php_phongo_utcdatetime_init_from_date(php_phongo_utcdatetime_t* intern, php_date_obj* datetime_obj) /* {{{ */
 {
 	int64_t sec, usec;
 
@@ -135,7 +107,7 @@ static bool php_phongo_utcdatetime_init_from_date(php_phongo_utcdatetime_t *inte
 #endif
 
 	intern->milliseconds = (sec * 1000) + (usec / 1000);
-	intern->initialized = true;
+	intern->initialized  = true;
 
 	return true;
 } /* }}} */
@@ -146,9 +118,9 @@ static bool php_phongo_utcdatetime_init_from_date(php_phongo_utcdatetime_t *inte
    current time. */
 static PHP_METHOD(UTCDateTime, __construct)
 {
-	php_phongo_utcdatetime_t    *intern;
-	zend_error_handling          error_handling;
-	zval                        *milliseconds = NULL;
+	php_phongo_utcdatetime_t* intern;
+	zend_error_handling       error_handling;
+	zval*                     milliseconds = NULL;
 
 	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling TSRMLS_CC);
 	intern = Z_UTCDATETIME_OBJ_P(getThis());
@@ -166,7 +138,8 @@ static PHP_METHOD(UTCDateTime, __construct)
 
 	if (Z_TYPE_P(milliseconds) == IS_OBJECT) {
 		if (instanceof_function(Z_OBJCE_P(milliseconds), php_date_get_date_ce() TSRMLS_CC) ||
-		    (php_phongo_date_immutable_ce && instanceof_function(Z_OBJCE_P(milliseconds), php_phongo_date_immutable_ce TSRMLS_CC))) {
+			(php_phongo_date_immutable_ce && instanceof_function(Z_OBJCE_P(milliseconds), php_phongo_date_immutable_ce TSRMLS_CC))) {
+
 			php_phongo_utcdatetime_init_from_date(intern, Z_PHPDATE_P(milliseconds));
 		} else {
 			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Expected instance of DateTimeInterface, %s given", ZSTR_VAL(Z_OBJCE_P(milliseconds)->name));
@@ -181,7 +154,7 @@ static PHP_METHOD(UTCDateTime, __construct)
 
 	if (Z_TYPE_P(milliseconds) == IS_DOUBLE) {
 		char tmp[24];
-		int tmp_len;
+		int  tmp_len;
 
 		tmp_len = snprintf(tmp, sizeof(tmp), "%.0f", Z_DVAL_P(milliseconds) > 0 ? floor(Z_DVAL_P(milliseconds)) : ceil(Z_DVAL_P(milliseconds)));
 
@@ -201,9 +174,9 @@ static PHP_METHOD(UTCDateTime, __construct)
 */
 static PHP_METHOD(UTCDateTime, __set_state)
 {
-	php_phongo_utcdatetime_t *intern;
-	HashTable                *props;
-	zval                     *array;
+	php_phongo_utcdatetime_t* intern;
+	HashTable*                props;
+	zval*                     array;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a", &array) == FAILURE) {
 		RETURN_FALSE;
@@ -212,7 +185,7 @@ static PHP_METHOD(UTCDateTime, __set_state)
 	object_init_ex(return_value, php_phongo_utcdatetime_ce);
 
 	intern = Z_UTCDATETIME_OBJ_P(return_value);
-	props = Z_ARRVAL_P(array);
+	props  = Z_ARRVAL_P(array);
 
 	php_phongo_utcdatetime_init_from_hash(intern, props TSRMLS_CC);
 } /* }}} */
@@ -221,10 +194,7 @@ static PHP_METHOD(UTCDateTime, __set_state)
    Returns the UTCDateTime's milliseconds as a string */
 static PHP_METHOD(UTCDateTime, __toString)
 {
-	php_phongo_utcdatetime_t    *intern;
-	char *tmp;
-	int tmp_len;
-
+	php_phongo_utcdatetime_t* intern;
 
 	intern = Z_UTCDATETIME_OBJ_P(getThis());
 
@@ -232,21 +202,18 @@ static PHP_METHOD(UTCDateTime, __toString)
 		return;
 	}
 
-	tmp_len = spprintf(&tmp, 0, "%" PRId64, intern->milliseconds);
-	PHONGO_RETVAL_STRINGL(tmp, tmp_len);
-	efree(tmp);
+	ZVAL_INT64_STRING(return_value, intern->milliseconds);
 } /* }}} */
 
 /* {{{ proto DateTime MongoDB\BSON\UTCDateTime::toDateTime()
    Returns a DateTime object representing this UTCDateTime */
 static PHP_METHOD(UTCDateTime, toDateTime)
 {
-	php_phongo_utcdatetime_t *intern;
-	php_date_obj             *datetime_obj;
-	char                     *sec;
+	php_phongo_utcdatetime_t* intern;
+	php_date_obj*             datetime_obj;
+	char*                     sec;
 	size_t                    sec_len;
 
-
 	intern = Z_UTCDATETIME_OBJ_P(getThis());
 
 	if (zend_parse_parameters_none() == FAILURE) {
@@ -272,9 +239,7 @@ static PHP_METHOD(UTCDateTime, toDateTime)
 */
 static PHP_METHOD(UTCDateTime, jsonSerialize)
 {
-	php_phongo_utcdatetime_t *intern;
-	char                      s_milliseconds[24];
-	int                       s_milliseconds_len;
+	php_phongo_utcdatetime_t* intern;
 
 	if (zend_parse_parameters_none() == FAILURE) {
 		return;
@@ -282,44 +247,25 @@ static PHP_METHOD(UTCDateTime, jsonSerialize)
 
 	intern = Z_UTCDATETIME_OBJ_P(getThis());
 
-	s_milliseconds_len = snprintf(s_milliseconds, sizeof(s_milliseconds), "%" PRId64, intern->milliseconds);
-
 	array_init_size(return_value, 1);
 
-#if PHP_VERSION_ID >= 70000
 	{
 		zval udt;
 
 		array_init_size(&udt, 1);
-		ADD_ASSOC_STRINGL(&udt, "$numberLong", s_milliseconds, s_milliseconds_len);
+		ADD_ASSOC_INT64_AS_STRING(&udt, "$numberLong", intern->milliseconds);
 		ADD_ASSOC_ZVAL_EX(return_value, "$date", &udt);
 	}
-#else
-	{
-		zval *udt;
-
-		MAKE_STD_ZVAL(udt);
-		array_init_size(udt, 1);
-		ADD_ASSOC_STRINGL(udt, "$numberLong", s_milliseconds, s_milliseconds_len);
-		ADD_ASSOC_ZVAL_EX(return_value, "$date", udt);
-	}
-#endif
 } /* }}} */
 
 /* {{{ proto string MongoDB\BSON\UTCDateTime::serialize()
 */
 static PHP_METHOD(UTCDateTime, serialize)
 {
-	php_phongo_utcdatetime_t *intern;
-#if PHP_VERSION_ID >= 70000
+	php_phongo_utcdatetime_t* intern;
 	zval                      retval;
-#else
-	zval                     *retval;
-#endif
 	php_serialize_data_t      var_hash;
 	smart_str                 buf = { 0 };
-	char                      s_milliseconds[24];
-	int                       s_milliseconds_len;
 
 	intern = Z_UTCDATETIME_OBJ_P(getThis());
 
@@ -327,16 +273,8 @@ static PHP_METHOD(UTCDateTime, serialize)
 		return;
 	}
 
-	s_milliseconds_len = snprintf(s_milliseconds, sizeof(s_milliseconds), "%" PRId64, intern->milliseconds);
-
-#if PHP_VERSION_ID >= 70000
-	array_init_size(&retval, 2);
-	ADD_ASSOC_STRINGL(&retval, "milliseconds", s_milliseconds, s_milliseconds_len);
-#else
-	ALLOC_INIT_ZVAL(retval);
-	array_init_size(retval, 2);
-	ADD_ASSOC_STRINGL(retval, "milliseconds", s_milliseconds, s_milliseconds_len);
-#endif
+	array_init_size(&retval, 1);
+	ADD_ASSOC_INT64_AS_STRING(&retval, "milliseconds", intern->milliseconds);
 
 	PHP_VAR_SERIALIZE_INIT(var_hash);
 	php_var_serialize(&buf, &retval, &var_hash TSRMLS_CC);
@@ -353,15 +291,11 @@ static PHP_METHOD(UTCDateTime, serialize)
 */
 static PHP_METHOD(UTCDateTime, unserialize)
 {
-	php_phongo_utcdatetime_t *intern;
+	php_phongo_utcdatetime_t* intern;
 	zend_error_handling       error_handling;
-	char                     *serialized;
-	phongo_zpp_char_len       serialized_len;
-#if PHP_VERSION_ID >= 70000
+	char*                     serialized;
+	size_t                    serialized_len;
 	zval                      props;
-#else
-	zval                     *props;
-#endif
 	php_unserialize_data_t    var_hash;
 
 	intern = Z_UTCDATETIME_OBJ_P(getThis());
@@ -374,11 +308,8 @@ static PHP_METHOD(UTCDateTime, unserialize)
 	}
 	zend_restore_error_handling(&error_handling TSRMLS_CC);
 
-#if PHP_VERSION_ID < 70000
-	ALLOC_INIT_ZVAL(props);
-#endif
 	PHP_VAR_UNSERIALIZE_INIT(var_hash);
-	if (!php_var_unserialize(&props, (const unsigned char**) &serialized, (unsigned char *) serialized + serialized_len, &var_hash TSRMLS_CC)) {
+	if (!php_var_unserialize(&props, (const unsigned char**) &serialized, (unsigned char*) serialized + serialized_len, &var_hash TSRMLS_CC)) {
 		zval_ptr_dtor(&props);
 		phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "%s unserialization failed", ZSTR_VAL(php_phongo_utcdatetime_ce->name));
 
@@ -387,11 +318,7 @@ static PHP_METHOD(UTCDateTime, unserialize)
 	}
 	PHP_VAR_UNSERIALIZE_DESTROY(var_hash);
 
-#if PHP_VERSION_ID >= 70000
 	php_phongo_utcdatetime_init_from_hash(intern, HASH_OF(&props) TSRMLS_CC);
-#else
-	php_phongo_utcdatetime_init_from_hash(intern, HASH_OF(props) TSRMLS_CC);
-#endif
 	zval_ptr_dtor(&props);
 } /* }}} */
 
@@ -412,23 +339,25 @@ ZEND_BEGIN_ARG_INFO_EX(ai_UTCDateTime_void, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_utcdatetime_me[] = {
-	PHP_ME(UTCDateTime, __construct, ai_UTCDateTime___construct, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(UTCDateTime, __set_state, ai_UTCDateTime___set_state, ZEND_ACC_PUBLIC|ZEND_ACC_STATIC)
-	PHP_ME(UTCDateTime, __toString, ai_UTCDateTime_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(UTCDateTime, jsonSerialize, ai_UTCDateTime_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(UTCDateTime, serialize, ai_UTCDateTime_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(UTCDateTime, unserialize, ai_UTCDateTime_unserialize, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(UTCDateTime, toDateTime, ai_UTCDateTime_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	/* clang-format off */
+	PHP_ME(UTCDateTime, __construct, ai_UTCDateTime___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(UTCDateTime, __set_state, ai_UTCDateTime___set_state, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+	PHP_ME(UTCDateTime, __toString, ai_UTCDateTime_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(UTCDateTime, jsonSerialize, ai_UTCDateTime_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(UTCDateTime, serialize, ai_UTCDateTime_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(UTCDateTime, unserialize, ai_UTCDateTime_unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(UTCDateTime, toDateTime, ai_UTCDateTime_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_FE_END
+	/* clang-format on */
 };
 /* }}} */
 
 /* {{{ MongoDB\BSON\UTCDateTime object handlers */
 static zend_object_handlers php_phongo_handler_utcdatetime;
 
-static void php_phongo_utcdatetime_free_object(phongo_free_object_arg *object TSRMLS_DC) /* {{{ */
+static void php_phongo_utcdatetime_free_object(zend_object* object TSRMLS_DC) /* {{{ */
 {
-	php_phongo_utcdatetime_t *intern = Z_OBJ_UTCDATETIME(object);
+	php_phongo_utcdatetime_t* intern = Z_OBJ_UTCDATETIME(object);
 
 	zend_object_std_dtor(&intern->std TSRMLS_CC);
 
@@ -436,37 +365,40 @@ static void php_phongo_utcdatetime_free_object(phongo_free_object_arg *object TS
 		zend_hash_destroy(intern->properties);
 		FREE_HASHTABLE(intern->properties);
 	}
-
-#if PHP_VERSION_ID < 70000
-	efree(intern);
-#endif
 } /* }}} */
 
-static phongo_create_object_retval php_phongo_utcdatetime_create_object(zend_class_entry *class_type TSRMLS_DC) /* {{{ */
+static zend_object* php_phongo_utcdatetime_create_object(zend_class_entry* class_type TSRMLS_DC) /* {{{ */
 {
-	php_phongo_utcdatetime_t *intern = NULL;
+	php_phongo_utcdatetime_t* intern = NULL;
 
 	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_utcdatetime_t, class_type);
 
 	zend_object_std_init(&intern->std, class_type TSRMLS_CC);
 	object_properties_init(&intern->std, class_type);
 
-#if PHP_VERSION_ID >= 70000
 	intern->std.handlers = &php_phongo_handler_utcdatetime;
 
 	return &intern->std;
-#else
-	{
-		zend_object_value retval;
-		retval.handle = zend_objects_store_put(intern, (zend_objects_store_dtor_t) zend_objects_destroy_object, php_phongo_utcdatetime_free_object, NULL TSRMLS_CC);
-		retval.handlers = &php_phongo_handler_utcdatetime;
-
-		return retval;
-	}
-#endif
 } /* }}} */
 
-static int php_phongo_utcdatetime_compare_objects(zval *o1, zval *o2 TSRMLS_DC) /* {{{ */
+static zend_object* php_phongo_utcdatetime_clone_object(zval* object TSRMLS_DC) /* {{{ */
+{
+	php_phongo_utcdatetime_t* intern;
+	php_phongo_utcdatetime_t* new_intern;
+	zend_object*              new_object;
+
+	intern     = Z_UTCDATETIME_OBJ_P(object);
+	new_object = php_phongo_utcdatetime_create_object(Z_OBJCE_P(object) TSRMLS_CC);
+
+	new_intern = Z_OBJ_UTCDATETIME(new_object);
+	zend_objects_clone_members(&new_intern->std, &intern->std TSRMLS_CC);
+
+	php_phongo_utcdatetime_init(new_intern, intern->milliseconds);
+
+	return new_object;
+} /* }}} */
+
+static int php_phongo_utcdatetime_compare_objects(zval* o1, zval* o2 TSRMLS_DC) /* {{{ */
 {
 	php_phongo_utcdatetime_t *intern1, *intern2;
 
@@ -480,58 +412,36 @@ static int php_phongo_utcdatetime_compare_objects(zval *o1, zval *o2 TSRMLS_DC)
 	return 0;
 } /* }}} */
 
-static HashTable *php_phongo_utcdatetime_get_gc(zval *object, phongo_get_gc_table table, int *n TSRMLS_DC) /* {{{ */
+static HashTable* php_phongo_utcdatetime_get_properties_hash(zval* object, bool is_debug TSRMLS_DC) /* {{{ */
 {
-	*table = NULL;
-	*n = 0;
-
-	return Z_UTCDATETIME_OBJ_P(object)->properties;
-} /* }}} */
-
-static HashTable *php_phongo_utcdatetime_get_properties_hash(zval *object, bool is_debug TSRMLS_DC) /* {{{ */
-{
-	php_phongo_utcdatetime_t *intern;
-	HashTable                *props;
-	char                      s_milliseconds[24];
-	int                       s_milliseconds_len;
+	php_phongo_utcdatetime_t* intern;
+	HashTable*                props;
 
 	intern = Z_UTCDATETIME_OBJ_P(object);
 
-	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, 2);
+	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, 1);
 
 	if (!intern->initialized) {
 		return props;
 	}
 
-	s_milliseconds_len = snprintf(s_milliseconds, sizeof(s_milliseconds), "%" PRId64, intern->milliseconds);
-
-#if PHP_VERSION_ID >= 70000
 	{
 		zval milliseconds;
 
-		ZVAL_STRINGL(&milliseconds, s_milliseconds, s_milliseconds_len);
-		zend_hash_str_update(props, "milliseconds", sizeof("milliseconds")-1, &milliseconds);
+		ZVAL_INT64_STRING(&milliseconds, intern->milliseconds);
+		zend_hash_str_update(props, "milliseconds", sizeof("milliseconds") - 1, &milliseconds);
 	}
-#else
-	{
-		zval *milliseconds;
-
-		MAKE_STD_ZVAL(milliseconds);
-		ZVAL_STRINGL(milliseconds, s_milliseconds, s_milliseconds_len, 1);
-		zend_hash_update(props, "milliseconds", sizeof("milliseconds"), &milliseconds, sizeof(milliseconds), NULL);
-	}
-#endif
 
 	return props;
 } /* }}} */
 
-static HashTable *php_phongo_utcdatetime_get_debug_info(zval *object, int *is_temp TSRMLS_DC) /* {{{ */
+static HashTable* php_phongo_utcdatetime_get_debug_info(zval* object, int* is_temp TSRMLS_DC) /* {{{ */
 {
 	*is_temp = 1;
 	return php_phongo_utcdatetime_get_properties_hash(object, true TSRMLS_CC);
 } /* }}} */
 
-static HashTable *php_phongo_utcdatetime_get_properties(zval *object TSRMLS_DC) /* {{{ */
+static HashTable* php_phongo_utcdatetime_get_properties(zval* object TSRMLS_DC) /* {{{ */
 {
 	return php_phongo_utcdatetime_get_properties_hash(object, false TSRMLS_CC);
 } /* }}} */
@@ -542,7 +452,7 @@ void php_phongo_utcdatetime_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	zend_class_entry ce;
 
 	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\BSON", "UTCDateTime", php_phongo_utcdatetime_me);
-	php_phongo_utcdatetime_ce = zend_register_internal_class(&ce TSRMLS_CC);
+	php_phongo_utcdatetime_ce                = zend_register_internal_class(&ce TSRMLS_CC);
 	php_phongo_utcdatetime_ce->create_object = php_phongo_utcdatetime_create_object;
 	PHONGO_CE_FINAL(php_phongo_utcdatetime_ce);
 
@@ -552,14 +462,12 @@ void php_phongo_utcdatetime_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	zend_class_implements(php_phongo_utcdatetime_ce TSRMLS_CC, 1, zend_ce_serializable);
 
 	memcpy(&php_phongo_handler_utcdatetime, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
+	php_phongo_handler_utcdatetime.clone_obj       = php_phongo_utcdatetime_clone_object;
 	php_phongo_handler_utcdatetime.compare_objects = php_phongo_utcdatetime_compare_objects;
-	php_phongo_handler_utcdatetime.get_debug_info = php_phongo_utcdatetime_get_debug_info;
-	php_phongo_handler_utcdatetime.get_gc = php_phongo_utcdatetime_get_gc;
-	php_phongo_handler_utcdatetime.get_properties = php_phongo_utcdatetime_get_properties;
-#if PHP_VERSION_ID >= 70000
-	php_phongo_handler_utcdatetime.free_obj = php_phongo_utcdatetime_free_object;
-	php_phongo_handler_utcdatetime.offset = XtOffsetOf(php_phongo_utcdatetime_t, std);
-#endif
+	php_phongo_handler_utcdatetime.get_debug_info  = php_phongo_utcdatetime_get_debug_info;
+	php_phongo_handler_utcdatetime.get_properties  = php_phongo_utcdatetime_get_properties;
+	php_phongo_handler_utcdatetime.free_obj        = php_phongo_utcdatetime_free_object;
+	php_phongo_handler_utcdatetime.offset          = XtOffsetOf(php_phongo_utcdatetime_t, std);
 } /* }}} */
 
 /*
