@@ -14,13 +14,13 @@
  * limitations under the License.
  */
 
+#include <php.h>
+#include <Zend/zend_interfaces.h>
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <php.h>
-#include <Zend/zend_interfaces.h>
-
 #include "phongo_compat.h"
 #include "php_phongo.h"
 #include "php_bson.h"
@@ -29,28 +29,24 @@
 
 zend_class_entry* php_phongo_session_ce;
 
-#define SESSION_CHECK_LIVELINESS(i,m) \
-	if (!(i)->client_session) { \
-		phongo_throw_exception(PHONGO_ERROR_LOGIC TSRMLS_CC, "Cannot call '%s', as the session has already been ended.", (m)); \
-		return; \
+#define PHONGO_TRANSACTION_NONE "none"
+#define PHONGO_TRANSACTION_STARTING "starting"
+#define PHONGO_TRANSACTION_IN_PROGRESS "in_progress"
+#define PHONGO_TRANSACTION_COMMITTED "committed"
+#define PHONGO_TRANSACTION_ABORTED "aborted"
+
+#define SESSION_CHECK_LIVELINESS(i, m)                                  \
+	if (!(i)->client_session) {                                         \
+		phongo_throw_exception(                                         \
+			PHONGO_ERROR_LOGIC,                                         \
+			"Cannot call '%s', as the session has already been ended.", \
+			(m));                                                       \
+		return;                                                         \
 	}
 
-static bool php_phongo_topology_is_sharded_cluster(mongoc_client_t* client)
+static bool php_phongo_session_get_timestamp_parts(zval* obj, uint32_t* timestamp, uint32_t* increment)
 {
-	mongoc_server_description_t* sd;
-	bool ret;
-
-	sd = mongoc_client_select_server(client, true, NULL, NULL);
-	ret = (sd && !strcmp(mongoc_server_description_type(sd), php_phongo_server_description_type_map[PHONGO_SERVER_MONGOS].name));
-	mongoc_server_description_destroy(sd);
-
-	return ret;
-}
-
-static bool php_phongo_session_get_timestamp_parts(zval* obj, uint32_t* timestamp, uint32_t* increment TSRMLS_DC)
-{
-	bool retval = false;
-#if PHP_VERSION_ID >= 70000
+	bool retval     = false;
 	zval ztimestamp = ZVAL_STATIC_INIT;
 	zval zincrement = ZVAL_STATIC_INIT;
 
@@ -68,25 +64,6 @@ static bool php_phongo_session_get_timestamp_parts(zval* obj, uint32_t* timestam
 
 	*timestamp = Z_LVAL(ztimestamp);
 	*increment = Z_LVAL(zincrement);
-#else
-	zval* ztimestamp = NULL;
-	zval* zincrement = NULL;
-
-	zend_call_method_with_0_params(&obj, NULL, NULL, "getTimestamp", &ztimestamp);
-
-	if (Z_ISUNDEF(ztimestamp) || EG(exception)) {
-		goto cleanup;
-	}
-
-	zend_call_method_with_0_params(&obj, NULL, NULL, "getIncrement", &zincrement);
-
-	if (Z_ISUNDEF(zincrement) || EG(exception)) {
-		goto cleanup;
-	}
-
-	*timestamp = Z_LVAL_P(ztimestamp);
-	*increment = Z_LVAL_P(zincrement);
-#endif
 
 	retval = true;
 
@@ -102,6 +79,25 @@ cleanup:
 	return retval;
 }
 
+static const char* php_phongo_get_transaction_state_string(mongoc_transaction_state_t state)
+{
+	switch (state) {
+		case MONGOC_TRANSACTION_NONE:
+			return PHONGO_TRANSACTION_NONE;
+		case MONGOC_TRANSACTION_STARTING:
+			return PHONGO_TRANSACTION_STARTING;
+		case MONGOC_TRANSACTION_IN_PROGRESS:
+			return PHONGO_TRANSACTION_IN_PROGRESS;
+		case MONGOC_TRANSACTION_COMMITTED:
+			return PHONGO_TRANSACTION_COMMITTED;
+		case MONGOC_TRANSACTION_ABORTED:
+			return PHONGO_TRANSACTION_ABORTED;
+		default:
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Invalid transaction state %d given", (int) state);
+			return NULL;
+	}
+}
+
 /* {{{ proto void MongoDB\Driver\Session::advanceClusterTime(array|object $clusterTime)
    Advances the cluster time for this Session */
 static PHP_METHOD(Session, advanceClusterTime)
@@ -109,17 +105,15 @@ static PHP_METHOD(Session, advanceClusterTime)
 	php_phongo_session_t* intern;
 	zval*                 zcluster_time;
 	bson_t                cluster_time = BSON_INITIALIZER;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "advanceClusterTime")
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "A", &zcluster_time) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "A", &zcluster_time) == FAILURE) {
 		return;
 	}
 
-	php_phongo_zval_to_bson(zcluster_time, PHONGO_BSON_NONE, &cluster_time, NULL TSRMLS_CC);
+	php_phongo_zval_to_bson(zcluster_time, PHONGO_BSON_NONE, &cluster_time, NULL);
 
 	/* An exception may be thrown during BSON conversion */
 	if (EG(exception)) {
@@ -140,17 +134,15 @@ static PHP_METHOD(Session, advanceOperationTime)
 	zval*                 ztimestamp;
 	uint32_t              timestamp = 0;
 	uint32_t              increment = 0;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "advanceOperationTime")
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "O", &ztimestamp, php_phongo_timestamp_interface_ce) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &ztimestamp, php_phongo_timestamp_interface_ce) == FAILURE) {
 		return;
 	}
 
-	if (!php_phongo_session_get_timestamp_parts(ztimestamp, &timestamp, &increment TSRMLS_CC)) {
+	if (!php_phongo_session_get_timestamp_parts(ztimestamp, &timestamp, &increment)) {
 		return;
 	}
 
@@ -163,9 +155,9 @@ static PHP_METHOD(Session, getClusterTime)
 {
 	php_phongo_session_t* intern;
 	const bson_t*         cluster_time;
-	php_phongo_bson_state state = PHONGO_BSON_STATE_INITIALIZER;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
+	php_phongo_bson_state state;
+
+	PHONGO_BSON_INIT_STATE(state);
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getClusterTime")
@@ -186,11 +178,7 @@ static PHP_METHOD(Session, getClusterTime)
 		return;
 	}
 
-#if PHP_VERSION_ID >= 70000
 	RETURN_ZVAL(&state.zchild, 0, 1);
-#else
-	RETURN_ZVAL(state.zchild, 0, 1);
-#endif
 } /* }}} */
 
 /* {{{ proto object MongoDB\Driver\Session::getLogicalSessionId()
@@ -199,9 +187,9 @@ static PHP_METHOD(Session, getLogicalSessionId)
 {
 	php_phongo_session_t* intern;
 	const bson_t*         lsid;
-	php_phongo_bson_state state = PHONGO_BSON_STATE_INITIALIZER;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
+	php_phongo_bson_state state;
+
+	PHONGO_BSON_INIT_STATE(state);
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getLogicalSessionId")
@@ -218,11 +206,7 @@ static PHP_METHOD(Session, getLogicalSessionId)
 		return;
 	}
 
-#if PHP_VERSION_ID >= 70000
 	RETURN_ZVAL(&state.zchild, 0, 1);
-#else
-	RETURN_ZVAL(state.zchild, 0, 1);
-#endif
 } /* }}} */
 
 /* {{{ proto MongoDB\BSON\Timestamp|null MongoDB\Driver\Session::getOperationTime()
@@ -231,8 +215,6 @@ static PHP_METHOD(Session, getOperationTime)
 {
 	php_phongo_session_t* intern;
 	uint32_t              timestamp, increment;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getOperationTime")
@@ -250,22 +232,137 @@ static PHP_METHOD(Session, getOperationTime)
 		RETURN_NULL();
 	}
 
-	php_phongo_new_timestamp_from_increment_and_timestamp(return_value, increment, timestamp TSRMLS_CC);
+	php_phongo_bson_new_timestamp_from_increment_and_timestamp(return_value, increment, timestamp);
+} /* }}} */
+
+/* {{{ proto MongoDB\Driver\Server|null MongoDB\Driver\Session::getServer()
+   Returns the server this session is pinned to */
+static PHP_METHOD(Session, getServer)
+{
+	php_phongo_session_t* intern;
+	uint32_t              server_id = 0;
+
+	intern = Z_SESSION_OBJ_P(getThis());
+	SESSION_CHECK_LIVELINESS(intern, "getServer")
+
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
+
+	server_id = mongoc_client_session_get_server_id(intern->client_session);
+
+	/* For sessions without a pinned server, 0 is returned. */
+	if (!server_id) {
+		RETURN_NULL();
+	}
+
+	phongo_server_init(return_value, intern->client, server_id);
+} /* }}} */
+
+/* {{{ proto array|null MongoDB\Driver\Session::getTransactionOptions()
+   Returns options for the currently running transaction */
+static PHP_METHOD(Session, getTransactionOptions)
+{
+	php_phongo_session_t*         intern;
+	mongoc_transaction_opt_t*     opts;
+	int64_t                       max_commit_time_ms;
+	const mongoc_read_concern_t*  read_concern;
+	const mongoc_read_prefs_t*    read_preference;
+	const mongoc_write_concern_t* write_concern;
+
+	intern = Z_SESSION_OBJ_P(getThis());
+	SESSION_CHECK_LIVELINESS(intern, "getTransactionOptions")
+
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
+
+	opts = mongoc_session_opts_get_transaction_opts(intern->client_session);
+
+	if (!opts) {
+		return;
+	}
+
+	max_commit_time_ms = mongoc_transaction_opts_get_max_commit_time_ms(opts);
+	read_concern       = mongoc_transaction_opts_get_read_concern(opts);
+	read_preference    = mongoc_transaction_opts_get_read_prefs(opts);
+	write_concern      = mongoc_transaction_opts_get_write_concern(opts);
+
+	array_init_size(return_value, 4);
+
+	if (max_commit_time_ms) {
+		ADD_ASSOC_LONG_EX(return_value, "maxCommitTimeMS", max_commit_time_ms);
+	}
+
+	if (!mongoc_read_concern_is_default(read_concern)) {
+		zval zread_concern;
+
+		phongo_readconcern_init(&zread_concern, read_concern);
+		ADD_ASSOC_ZVAL_EX(return_value, "readConcern", &zread_concern);
+	}
+
+	if (read_preference) {
+		zval zread_preference;
+
+		phongo_readpreference_init(&zread_preference, read_preference);
+		ADD_ASSOC_ZVAL_EX(return_value, "readPreference", &zread_preference);
+	}
+
+	if (!mongoc_write_concern_is_default(write_concern)) {
+		zval zwrite_concern;
+
+		phongo_writeconcern_init(&zwrite_concern, write_concern);
+		ADD_ASSOC_ZVAL_EX(return_value, "writeConcern", &zwrite_concern);
+	}
+} /* }}} */
+
+/* {{{ proto string MongoDB\Driver\Session::getTransactionState()
+   Returns the current transaction state for this session */
+static PHP_METHOD(Session, getTransactionState)
+{
+	php_phongo_session_t* intern;
+	const char*           state;
+
+	intern = Z_SESSION_OBJ_P(getThis());
+	SESSION_CHECK_LIVELINESS(intern, "getTransactionState")
+
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
+
+	state = php_phongo_get_transaction_state_string(mongoc_client_session_get_transaction_state(intern->client_session));
+	if (!state) {
+		/* Exception already thrown */
+		return;
+	}
+
+	RETURN_STRING(state);
 } /* }}} */
 
 /* Creates a opts structure from an array optionally containing an RP, RC,
- * and/or WC object. Returns NULL if no options were found, or there was an
- * invalid option. If there was an invalid option or structure, an exception
- * will be thrown too. */
-mongoc_transaction_opt_t* php_mongodb_session_parse_transaction_options(zval* options TSRMLS_DC)
+ * WC object, and/or maxCommitTimeMS int. Returns NULL if no options were found,
+ * or there was an invalid option. If there was an invalid option or structure,
+ * an exception will be thrown too. */
+mongoc_transaction_opt_t* php_mongodb_session_parse_transaction_options(zval* options)
 {
 	mongoc_transaction_opt_t* opts = NULL;
 
-	if (php_array_existsc(options, "readConcern")) {
-		zval* read_concern = php_array_fetchc(options, "readConcern");
+	if (php_array_existsc(options, "maxCommitTimeMS")) {
+		int64_t max_commit_time_ms = php_array_fetchc_long(options, "maxCommitTimeMS");
 
-		if (Z_TYPE_P(read_concern) != IS_OBJECT || !instanceof_function(Z_OBJCE_P(read_concern), php_phongo_readconcern_ce TSRMLS_CC)) {
-			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Expected \"readConcern\" option to be %s, %s given", ZSTR_VAL(php_phongo_readconcern_ce->name), PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(read_concern));
+		if (max_commit_time_ms < 0) {
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"maxCommitTimeMS\" option to be >= 0, %" PRId64 " given", max_commit_time_ms);
+			/* Freeing opts is not needed here, as it can't be set yet. The
+			 * code is here to keep it consistent with the others in case more
+			 * options are added before this one. */
+			if (opts) {
+				mongoc_transaction_opts_destroy(opts);
+			}
+			return NULL;
+		}
+
+		if (max_commit_time_ms > UINT32_MAX) {
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"maxCommitTimeMS\" option to be <= %" PRIu32 ", %" PRId64 " given", UINT32_MAX, max_commit_time_ms);
 			/* Freeing opts is not needed here, as it can't be set yet. The
 			 * code is here to keep it consistent with the others in case more
 			 * options are added before this one. */
@@ -279,14 +376,32 @@ mongoc_transaction_opt_t* php_mongodb_session_parse_transaction_options(zval* op
 			opts = mongoc_transaction_opts_new();
 		}
 
-		mongoc_transaction_opts_set_read_concern(opts, phongo_read_concern_from_zval(read_concern TSRMLS_CC));
+		mongoc_transaction_opts_set_max_commit_time_ms(opts, max_commit_time_ms);
+	}
+
+	if (php_array_existsc(options, "readConcern")) {
+		zval* read_concern = php_array_fetchc(options, "readConcern");
+
+		if (Z_TYPE_P(read_concern) != IS_OBJECT || !instanceof_function(Z_OBJCE_P(read_concern), php_phongo_readconcern_ce)) {
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"readConcern\" option to be %s, %s given", ZSTR_VAL(php_phongo_readconcern_ce->name), PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(read_concern));
+			if (opts) {
+				mongoc_transaction_opts_destroy(opts);
+			}
+			return NULL;
+		}
+
+		if (!opts) {
+			opts = mongoc_transaction_opts_new();
+		}
+
+		mongoc_transaction_opts_set_read_concern(opts, phongo_read_concern_from_zval(read_concern));
 	}
 
 	if (php_array_existsc(options, "readPreference")) {
 		zval* read_preference = php_array_fetchc(options, "readPreference");
 
-		if (Z_TYPE_P(read_preference) != IS_OBJECT || !instanceof_function(Z_OBJCE_P(read_preference), php_phongo_readpreference_ce TSRMLS_CC)) {
-			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Expected \"readPreference\" option to be %s, %s given", ZSTR_VAL(php_phongo_readpreference_ce->name), PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(read_preference));
+		if (Z_TYPE_P(read_preference) != IS_OBJECT || !instanceof_function(Z_OBJCE_P(read_preference), php_phongo_readpreference_ce)) {
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"readPreference\" option to be %s, %s given", ZSTR_VAL(php_phongo_readpreference_ce->name), PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(read_preference));
 			if (opts) {
 				mongoc_transaction_opts_destroy(opts);
 			}
@@ -297,14 +412,14 @@ mongoc_transaction_opt_t* php_mongodb_session_parse_transaction_options(zval* op
 			opts = mongoc_transaction_opts_new();
 		}
 
-		mongoc_transaction_opts_set_read_prefs(opts, phongo_read_preference_from_zval(read_preference TSRMLS_CC));
+		mongoc_transaction_opts_set_read_prefs(opts, phongo_read_preference_from_zval(read_preference));
 	}
 
 	if (php_array_existsc(options, "writeConcern")) {
 		zval* write_concern = php_array_fetchc(options, "writeConcern");
 
-		if (Z_TYPE_P(write_concern) != IS_OBJECT || !instanceof_function(Z_OBJCE_P(write_concern), php_phongo_writeconcern_ce TSRMLS_CC)) {
-			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Expected \"writeConcern\" option to be %s, %s given", ZSTR_VAL(php_phongo_writeconcern_ce->name), PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(write_concern));
+		if (Z_TYPE_P(write_concern) != IS_OBJECT || !instanceof_function(Z_OBJCE_P(write_concern), php_phongo_writeconcern_ce)) {
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"writeConcern\" option to be %s, %s given", ZSTR_VAL(php_phongo_writeconcern_ce->name), PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(write_concern));
 			if (opts) {
 				mongoc_transaction_opts_destroy(opts);
 			}
@@ -315,7 +430,7 @@ mongoc_transaction_opt_t* php_mongodb_session_parse_transaction_options(zval* op
 			opts = mongoc_transaction_opts_new();
 		}
 
-		mongoc_transaction_opts_set_write_concern(opts, phongo_write_concern_from_zval(write_concern TSRMLS_CC));
+		mongoc_transaction_opts_set_write_concern(opts, phongo_write_concern_from_zval(write_concern));
 	}
 
 	return opts;
@@ -329,30 +444,23 @@ static PHP_METHOD(Session, startTransaction)
 	zval*                     options     = NULL;
 	mongoc_transaction_opt_t* txn_options = NULL;
 	bson_error_t              error;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "startTransaction")
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|a", &options) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|a!", &options) == FAILURE) {
 		return;
 	}
 
 	if (options) {
-		txn_options = php_mongodb_session_parse_transaction_options(options TSRMLS_CC);
+		txn_options = php_mongodb_session_parse_transaction_options(options);
 	}
 	if (EG(exception)) {
 		return;
 	}
 
-	if (php_phongo_topology_is_sharded_cluster(mongoc_client_session_get_client(intern->client_session))) {
-		phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "PHP MongoDB driver %s does not support running multi-document transactions on sharded clusters", PHP_MONGODB_VERSION);
-		return;
-	}
-
 	if (!mongoc_client_session_start_transaction(intern->client_session, txn_options, &error)) {
-		phongo_throw_exception_from_bson_error_t(&error TSRMLS_CC);
+		phongo_throw_exception_from_bson_error_t(&error);
 	}
 
 	if (txn_options) {
@@ -367,8 +475,6 @@ static PHP_METHOD(Session, commitTransaction)
 	php_phongo_session_t* intern;
 	bson_error_t          error;
 	bson_t                reply;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "commitTransaction")
@@ -378,7 +484,7 @@ static PHP_METHOD(Session, commitTransaction)
 	}
 
 	if (!mongoc_client_session_commit_transaction(intern->client_session, &reply, &error)) {
-		phongo_throw_exception_from_bson_error_t_and_reply(&error, &reply TSRMLS_CC);
+		phongo_throw_exception_from_bson_error_t_and_reply(&error, &reply);
 		bson_destroy(&reply);
 	}
 } /* }}} */
@@ -389,8 +495,6 @@ static PHP_METHOD(Session, abortTransaction)
 {
 	php_phongo_session_t* intern;
 	bson_error_t          error;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "abortTransaction")
@@ -400,7 +504,7 @@ static PHP_METHOD(Session, abortTransaction)
 	}
 
 	if (!mongoc_client_session_abort_transaction(intern->client_session, &error)) {
-		phongo_throw_exception_from_bson_error_t(&error TSRMLS_CC);
+		phongo_throw_exception_from_bson_error_t(&error);
 	}
 } /* }}} */
 
@@ -409,8 +513,6 @@ static PHP_METHOD(Session, abortTransaction)
 static PHP_METHOD(Session, endSession)
 {
 	php_phongo_session_t* intern;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr)
-	SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_SESSION_OBJ_P(getThis());
 
@@ -422,6 +524,22 @@ static PHP_METHOD(Session, endSession)
 	intern->client_session = NULL;
 } /* }}} */
 
+/* {{{ proto void MongoDB\Driver\Session::isInTransaction(void)
+   Returns whether a multi-document transaction is in progress */
+static PHP_METHOD(Session, isInTransaction)
+{
+	php_phongo_session_t* intern;
+
+	intern = Z_SESSION_OBJ_P(getThis());
+	SESSION_CHECK_LIVELINESS(intern, "isInTransaction")
+
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
+
+	RETURN_BOOL(mongoc_client_session_in_transaction(intern->client_session));
+} /* }}} */
+
 /* {{{ MongoDB\Driver\Session function entries */
 ZEND_BEGIN_ARG_INFO_EX(ai_Session_advanceClusterTime, 0, 0, 1)
 	ZEND_ARG_INFO(0, clusterTime)
@@ -432,7 +550,7 @@ ZEND_BEGIN_ARG_INFO_EX(ai_Session_advanceOperationTime, 0, 0, 1)
 ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(ai_Session_startTransaction, 0, 0, 0)
-	ZEND_ARG_INFO(0, options)
+	ZEND_ARG_ARRAY_INFO(0, options, 1)
 ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(ai_Session_void, 0, 0, 0)
@@ -440,15 +558,19 @@ ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_session_me[] = {
 	/* clang-format off */
+	PHP_ME(Session, abortTransaction, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Session, advanceClusterTime, ai_Session_advanceClusterTime, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Session, advanceOperationTime, ai_Session_advanceOperationTime, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(Session, commitTransaction, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(Session, endSession, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Session, getClusterTime, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Session, getLogicalSessionId, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Session, getOperationTime, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(Session, getServer, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(Session, getTransactionOptions, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(Session, getTransactionState, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(Session, isInTransaction, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Session, startTransaction, ai_Session_startTransaction, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, commitTransaction, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, abortTransaction, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, endSession, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	ZEND_NAMED_ME(__construct, PHP_FN(MongoDB_disabled___construct), ai_Session_void, ZEND_ACC_PRIVATE | ZEND_ACC_FINAL)
 	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_FE_END
@@ -459,46 +581,41 @@ static zend_function_entry php_phongo_session_me[] = {
 /* {{{ MongoDB\Driver\Session object handlers */
 static zend_object_handlers php_phongo_handler_session;
 
-static void php_phongo_session_free_object(phongo_free_object_arg* object TSRMLS_DC) /* {{{ */
+static void php_phongo_session_free_object(zend_object* object) /* {{{ */
 {
 	php_phongo_session_t* intern = Z_OBJ_SESSION(object);
 
-	zend_object_std_dtor(&intern->std TSRMLS_CC);
+	zend_object_std_dtor(&intern->std);
+
+	/* If this Session was created in a different process, reset the client so
+	 * that its session pool is cleared and mongoc_client_session_destroy will
+	 * destroy the corresponding server session rather than return it to the
+	 * now-empty pool. This will ensure that we do not re-use a server session
+	 * (i.e. LSID) created by a parent process. */
+	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern);
 
 	if (intern->client_session) {
 		mongoc_client_session_destroy(intern->client_session);
 	}
-
-#if PHP_VERSION_ID < 70000
-	efree(intern);
-#endif
 } /* }}} */
 
-static phongo_create_object_retval php_phongo_session_create_object(zend_class_entry* class_type TSRMLS_DC) /* {{{ */
+static zend_object* php_phongo_session_create_object(zend_class_entry* class_type) /* {{{ */
 {
 	php_phongo_session_t* intern = NULL;
 
 	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_session_t, class_type);
 
-	zend_object_std_init(&intern->std, class_type TSRMLS_CC);
+	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
 
-#if PHP_VERSION_ID >= 70000
+	PHONGO_SET_CREATED_BY_PID(intern);
+
 	intern->std.handlers = &php_phongo_handler_session;
 
 	return &intern->std;
-#else
-	{
-		zend_object_value retval;
-		retval.handle   = zend_objects_store_put(intern, (zend_objects_store_dtor_t) zend_objects_destroy_object, php_phongo_session_free_object, NULL TSRMLS_CC);
-		retval.handlers = &php_phongo_handler_session;
-
-		return retval;
-	}
-#endif
 } /* }}} */
 
-static HashTable* php_phongo_session_get_debug_info(zval* object, int* is_temp TSRMLS_DC) /* {{{ */
+static HashTable* php_phongo_session_get_debug_info(zval* object, int* is_temp) /* {{{ */
 {
 	php_phongo_session_t*       intern = NULL;
 	const mongoc_session_opt_t* cs_opts;
@@ -510,22 +627,19 @@ static HashTable* php_phongo_session_get_debug_info(zval* object, int* is_temp T
 	array_init(&retval);
 
 	if (intern->client_session) {
-		const bson_t* lsid;
+		const bson_t*         lsid;
+		php_phongo_bson_state state;
 
-		php_phongo_bson_state state = PHONGO_BSON_STATE_INITIALIZER;
-		/* Use native arrays for debugging output */
-		state.map.root_type     = PHONGO_TYPEMAP_NATIVE_ARRAY;
-		state.map.document_type = PHONGO_TYPEMAP_NATIVE_ARRAY;
+		PHONGO_BSON_INIT_DEBUG_STATE(state);
 
 		lsid = mongoc_client_session_get_lsid(intern->client_session);
 
-		php_phongo_bson_to_zval_ex(bson_get_data(lsid), lsid->len, &state);
+		if (!php_phongo_bson_to_zval_ex(bson_get_data(lsid), lsid->len, &state)) {
+			zval_ptr_dtor(&state.zchild);
+			goto done;
+		}
 
-#if PHP_VERSION_ID >= 70000
 		ADD_ASSOC_ZVAL_EX(&retval, "logicalSessionId", &state.zchild);
-#else
-		ADD_ASSOC_ZVAL_EX(&retval, "logicalSessionId", state.zchild);
-#endif
 	} else {
 		ADD_ASSOC_NULL_EX(&retval, "logicalSessionId");
 	}
@@ -533,21 +647,19 @@ static HashTable* php_phongo_session_get_debug_info(zval* object, int* is_temp T
 	if (intern->client_session) {
 		const bson_t* cluster_time;
 
-		php_phongo_bson_state state = PHONGO_BSON_STATE_INITIALIZER;
-		/* Use native arrays for debugging output */
-		state.map.root_type     = PHONGO_TYPEMAP_NATIVE_ARRAY;
-		state.map.document_type = PHONGO_TYPEMAP_NATIVE_ARRAY;
-
 		cluster_time = mongoc_client_session_get_cluster_time(intern->client_session);
 
 		if (cluster_time) {
-			php_phongo_bson_to_zval_ex(bson_get_data(cluster_time), cluster_time->len, &state);
+			php_phongo_bson_state state;
+
+			PHONGO_BSON_INIT_DEBUG_STATE(state);
+
+			if (!php_phongo_bson_to_zval_ex(bson_get_data(cluster_time), cluster_time->len, &state)) {
+				zval_ptr_dtor(&state.zchild);
+				goto done;
+			}
 
-#if PHP_VERSION_ID >= 70000
 			ADD_ASSOC_ZVAL_EX(&retval, "clusterTime", &state.zchild);
-#else
-			ADD_ASSOC_ZVAL_EX(&retval, "clusterTime", state.zchild);
-#endif
 		} else {
 			ADD_ASSOC_NULL_EX(&retval, "clusterTime");
 		}
@@ -568,18 +680,10 @@ static HashTable* php_phongo_session_get_debug_info(zval* object, int* is_temp T
 		mongoc_client_session_get_operation_time(intern->client_session, &timestamp, &increment);
 
 		if (timestamp && increment) {
-#if PHP_VERSION_ID >= 70000
 			zval ztimestamp;
 
-			php_phongo_new_timestamp_from_increment_and_timestamp(&ztimestamp, increment, timestamp TSRMLS_CC);
+			php_phongo_bson_new_timestamp_from_increment_and_timestamp(&ztimestamp, increment, timestamp);
 			ADD_ASSOC_ZVAL_EX(&retval, "operationTime", &ztimestamp);
-#else
-			zval* ztimestamp;
-
-			MAKE_STD_ZVAL(ztimestamp);
-			php_phongo_new_timestamp_from_increment_and_timestamp(ztimestamp, increment, timestamp TSRMLS_CC);
-			ADD_ASSOC_ZVAL_EX(&retval, "operationTime", ztimestamp);
-#endif
 		} else {
 			ADD_ASSOC_NULL_EX(&retval, "operationTime");
 		}
@@ -587,6 +691,23 @@ static HashTable* php_phongo_session_get_debug_info(zval* object, int* is_temp T
 		ADD_ASSOC_NULL_EX(&retval, "operationTime");
 	}
 
+	if (intern->client_session) {
+		uint32_t server_id = mongoc_client_session_get_server_id(intern->client_session);
+
+		if (server_id) {
+
+			zval server;
+
+			phongo_server_init(&server, intern->client, server_id);
+			ADD_ASSOC_ZVAL_EX(&retval, "server", &server);
+		} else {
+			ADD_ASSOC_NULL_EX(&retval, "server");
+		}
+	} else {
+		ADD_ASSOC_NULL_EX(&retval, "server");
+	}
+
+done:
 	return Z_ARRVAL(retval);
 } /* }}} */
 /* }}} */
@@ -596,17 +717,21 @@ void php_phongo_session_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	zend_class_entry ce;
 
 	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "Session", php_phongo_session_me);
-	php_phongo_session_ce                = zend_register_internal_class(&ce TSRMLS_CC);
+	php_phongo_session_ce                = zend_register_internal_class(&ce);
 	php_phongo_session_ce->create_object = php_phongo_session_create_object;
 	PHONGO_CE_FINAL(php_phongo_session_ce);
 	PHONGO_CE_DISABLE_SERIALIZATION(php_phongo_session_ce);
 
 	memcpy(&php_phongo_handler_session, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	php_phongo_handler_session.get_debug_info = php_phongo_session_get_debug_info;
-#if PHP_VERSION_ID >= 70000
-	php_phongo_handler_session.free_obj = php_phongo_session_free_object;
-	php_phongo_handler_session.offset   = XtOffsetOf(php_phongo_session_t, std);
-#endif
+	php_phongo_handler_session.free_obj       = php_phongo_session_free_object;
+	php_phongo_handler_session.offset         = XtOffsetOf(php_phongo_session_t, std);
+
+	zend_declare_class_constant_string(php_phongo_session_ce, ZEND_STRL("TRANSACTION_NONE"), PHONGO_TRANSACTION_NONE);
+	zend_declare_class_constant_string(php_phongo_session_ce, ZEND_STRL("TRANSACTION_STARTING"), PHONGO_TRANSACTION_STARTING);
+	zend_declare_class_constant_string(php_phongo_session_ce, ZEND_STRL("TRANSACTION_IN_PROGRESS"), PHONGO_TRANSACTION_IN_PROGRESS);
+	zend_declare_class_constant_string(php_phongo_session_ce, ZEND_STRL("TRANSACTION_COMMITTED"), PHONGO_TRANSACTION_COMMITTED);
+	zend_declare_class_constant_string(php_phongo_session_ce, ZEND_STRL("TRANSACTION_ABORTED"), PHONGO_TRANSACTION_ABORTED);
 } /* }}} */
 
 /*
