@@ -14,28 +14,28 @@
  * limitations under the License.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include <bson.h>
+#include <bson/bson.h>
 
 #include <php.h>
 #include <Zend/zend_hash.h>
 #include <Zend/zend_interfaces.h>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "php_phongo.h"
 #include "php_bson.h"
 #include "phongo_compat.h"
 
-#if SIZEOF_PHONGO_LONG == 8
+#if SIZEOF_ZEND_LONG == 8
 #define BSON_APPEND_INT(b, key, keylen, val)    \
 	if (val > INT32_MAX || val < INT32_MIN) {   \
 		bson_append_int64(b, key, keylen, val); \
 	} else {                                    \
 		bson_append_int32(b, key, keylen, val); \
 	}
-#elif SIZEOF_PHONGO_LONG == 4
+#elif SIZEOF_ZEND_LONG == 4
 #define BSON_APPEND_INT(b, key, keylen, val) \
 	bson_append_int32(b, key, keylen, val)
 #else
@@ -46,12 +46,12 @@
 #define MONGOC_LOG_DOMAIN "PHONGO-BSON"
 
 /* Forwards declarations */
-static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, bson_t* bson, bson_t** bson_out TSRMLS_DC);
+static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, bson_t* bson, bson_t** bson_out);
 
 /* Determines whether the argument should be serialized as a BSON array or
  * document. IS_ARRAY is returned if the argument's keys are a sequence of
  * integers starting at zero; otherwise, IS_OBJECT is returned. */
-static int php_phongo_is_array_or_document(zval* val TSRMLS_DC) /* {{{ */
+static int php_phongo_is_array_or_document(zval* val) /* {{{ */
 {
 	HashTable* ht_data = HASH_OF(val);
 	int        count;
@@ -62,7 +62,6 @@ static int php_phongo_is_array_or_document(zval* val TSRMLS_DC) /* {{{ */
 
 	count = ht_data ? zend_hash_num_elements(ht_data) : 0;
 	if (count > 0) {
-#if PHP_VERSION_ID >= 70000
 		zend_string* key;
 		zend_ulong   index, idx;
 
@@ -79,31 +78,6 @@ static int php_phongo_is_array_or_document(zval* val TSRMLS_DC) /* {{{ */
 			idx++;
 		}
 		ZEND_HASH_FOREACH_END();
-#else
-		char*         key;
-		unsigned int  key_len;
-		unsigned long index     = 0;
-		unsigned long idx       = 0;
-		int           hash_type = 0;
-		HashPosition  pos;
-
-		zend_hash_internal_pointer_reset_ex(ht_data, &pos);
-		for (;; zend_hash_move_forward_ex(ht_data, &pos)) {
-			hash_type = zend_hash_get_current_key_ex(ht_data, &key, &key_len, &index, 0, &pos);
-			if (hash_type == HASH_KEY_NON_EXISTENT) {
-				break;
-			}
-
-			if (hash_type == HASH_KEY_IS_STRING) {
-				return IS_OBJECT;
-			} else {
-				if (index != idx) {
-					return IS_OBJECT;
-				}
-			}
-			idx++;
-		}
-#endif
 	} else {
 		return Z_TYPE_P(val);
 	}
@@ -116,27 +90,19 @@ static int php_phongo_is_array_or_document(zval* val TSRMLS_DC) /* {{{ */
  * will be appended as an embedded document. Other MongoDB\BSON\Type instances
  * will be appended as the appropriate BSON type. Other array or object values
  * will be appended as an embedded document. */
-static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, const char* key, long key_len, zval* object TSRMLS_DC) /* {{{ */
+static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, const char* key, long key_len, zval* object) /* {{{ */
 {
-	if (Z_TYPE_P(object) == IS_OBJECT && instanceof_function(Z_OBJCE_P(object), php_phongo_cursorid_ce TSRMLS_CC)) {
+	if (Z_TYPE_P(object) == IS_OBJECT && instanceof_function(Z_OBJCE_P(object), php_phongo_cursorid_ce)) {
 		bson_append_int64(bson, key, key_len, Z_CURSORID_OBJ_P(object)->id);
 		return;
 	}
 
-	if (Z_TYPE_P(object) == IS_OBJECT && instanceof_function(Z_OBJCE_P(object), php_phongo_type_ce TSRMLS_CC)) {
-		if (instanceof_function(Z_OBJCE_P(object), php_phongo_serializable_ce TSRMLS_CC)) {
-#if PHP_VERSION_ID >= 70000
-			zval obj_data;
-#else
-			zval* obj_data = NULL;
-#endif
+	if (Z_TYPE_P(object) == IS_OBJECT && instanceof_function(Z_OBJCE_P(object), php_phongo_type_ce)) {
+		if (instanceof_function(Z_OBJCE_P(object), php_phongo_serializable_ce)) {
+			zval   obj_data;
 			bson_t child;
 
-#if PHP_VERSION_ID >= 70000
 			zend_call_method_with_0_params(object, NULL, NULL, BSON_SERIALIZE_FUNC_NAME, &obj_data);
-#else
-			zend_call_method_with_0_params(&object, NULL, NULL, BSON_SERIALIZE_FUNC_NAME, &obj_data);
-#endif
 
 			if (Z_ISUNDEF(obj_data)) {
 				/* zend_call_method() failed or bsonSerialize() threw an
@@ -144,55 +110,29 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 				return;
 			}
 
-#if PHP_VERSION_ID >= 70000
-			if (Z_TYPE(obj_data) != IS_ARRAY && !(Z_TYPE(obj_data) == IS_OBJECT && instanceof_function(Z_OBJCE(obj_data), zend_standard_class_def TSRMLS_CC))) {
-				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC,
+			if (Z_TYPE(obj_data) != IS_ARRAY && !(Z_TYPE(obj_data) == IS_OBJECT && instanceof_function(Z_OBJCE(obj_data), zend_standard_class_def))) {
+				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE,
 									   "Expected %s::%s() to return an array or stdClass, %s given",
 									   ZSTR_VAL(Z_OBJCE_P(object)->name),
 									   BSON_SERIALIZE_FUNC_NAME,
 									   PHONGO_ZVAL_CLASS_OR_TYPE_NAME(obj_data));
 				zval_ptr_dtor(&obj_data);
-#else
-			if (Z_TYPE_P(obj_data) != IS_ARRAY && !(Z_TYPE_P(obj_data) == IS_OBJECT && instanceof_function(Z_OBJCE_P(obj_data), zend_standard_class_def TSRMLS_CC))) {
-				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC,
-									   "Expected %s::%s() to return an array or stdClass, %s given",
-									   ZSTR_VAL(Z_OBJCE_P(object)->name),
-									   BSON_SERIALIZE_FUNC_NAME,
-									   PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(obj_data));
-				zval_ptr_dtor(&obj_data);
-#endif
 
 				return;
 			}
 
 			/* Persistable objects must always be serialized as BSON documents;
 			 * otherwise, infer based on bsonSerialize()'s return value. */
-#if PHP_VERSION_ID >= 70000
-			if (instanceof_function(Z_OBJCE_P(object), php_phongo_persistable_ce TSRMLS_CC) || php_phongo_is_array_or_document(&obj_data TSRMLS_CC) == IS_OBJECT) {
-#else
-			if (instanceof_function(Z_OBJCE_P(object), php_phongo_persistable_ce TSRMLS_CC) || php_phongo_is_array_or_document(obj_data TSRMLS_CC) == IS_OBJECT) {
-#endif
+			if (instanceof_function(Z_OBJCE_P(object), php_phongo_persistable_ce) || php_phongo_is_array_or_document(&obj_data) == IS_OBJECT) {
 				bson_append_document_begin(bson, key, key_len, &child);
-				if (instanceof_function(Z_OBJCE_P(object), php_phongo_persistable_ce TSRMLS_CC)) {
-#if PHP_VERSION_ID >= 70000
+				if (instanceof_function(Z_OBJCE_P(object), php_phongo_persistable_ce)) {
 					bson_append_binary(&child, PHONGO_ODM_FIELD_NAME, -1, 0x80, (const uint8_t*) Z_OBJCE_P(object)->name->val, Z_OBJCE_P(object)->name->len);
-#else
-					bson_append_binary(&child, PHONGO_ODM_FIELD_NAME, -1, 0x80, (const uint8_t*) Z_OBJCE_P(object)->name, strlen(Z_OBJCE_P(object)->name));
-#endif
 				}
-#if PHP_VERSION_ID >= 70000
-				php_phongo_zval_to_bson_internal(&obj_data, field_path, flags, &child, NULL TSRMLS_CC);
-#else
-				php_phongo_zval_to_bson_internal(obj_data, field_path, flags, &child, NULL TSRMLS_CC);
-#endif
+				php_phongo_zval_to_bson_internal(&obj_data, field_path, flags, &child, NULL);
 				bson_append_document_end(bson, &child);
 			} else {
 				bson_append_array_begin(bson, key, key_len, &child);
-#if PHP_VERSION_ID >= 70000
-				php_phongo_zval_to_bson_internal(&obj_data, field_path, flags, &child, NULL TSRMLS_CC);
-#else
-				php_phongo_zval_to_bson_internal(obj_data, field_path, flags, &child, NULL TSRMLS_CC);
-#endif
+				php_phongo_zval_to_bson_internal(&obj_data, field_path, flags, &child, NULL);
 				bson_append_array_end(bson, &child);
 			}
 
@@ -200,7 +140,7 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 			return;
 		}
 
-		if (instanceof_function(Z_OBJCE_P(object), php_phongo_objectid_ce TSRMLS_CC)) {
+		if (instanceof_function(Z_OBJCE_P(object), php_phongo_objectid_ce)) {
 			bson_oid_t             oid;
 			php_phongo_objectid_t* intern = Z_OBJECTID_OBJ_P(object);
 
@@ -209,42 +149,42 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 			bson_append_oid(bson, key, key_len, &oid);
 			return;
 		}
-		if (instanceof_function(Z_OBJCE_P(object), php_phongo_utcdatetime_ce TSRMLS_CC)) {
+		if (instanceof_function(Z_OBJCE_P(object), php_phongo_utcdatetime_ce)) {
 			php_phongo_utcdatetime_t* intern = Z_UTCDATETIME_OBJ_P(object);
 
 			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding UTCDateTime");
 			bson_append_date_time(bson, key, key_len, intern->milliseconds);
 			return;
 		}
-		if (instanceof_function(Z_OBJCE_P(object), php_phongo_binary_ce TSRMLS_CC)) {
+		if (instanceof_function(Z_OBJCE_P(object), php_phongo_binary_ce)) {
 			php_phongo_binary_t* intern = Z_BINARY_OBJ_P(object);
 
 			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Binary");
 			bson_append_binary(bson, key, key_len, intern->type, (const uint8_t*) intern->data, (uint32_t) intern->data_len);
 			return;
 		}
-		if (instanceof_function(Z_OBJCE_P(object), php_phongo_decimal128_ce TSRMLS_CC)) {
+		if (instanceof_function(Z_OBJCE_P(object), php_phongo_decimal128_ce)) {
 			php_phongo_decimal128_t* intern = Z_DECIMAL128_OBJ_P(object);
 
 			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Decimal128");
 			bson_append_decimal128(bson, key, key_len, &intern->decimal);
 			return;
 		}
-		if (instanceof_function(Z_OBJCE_P(object), php_phongo_int64_ce TSRMLS_CC)) {
+		if (instanceof_function(Z_OBJCE_P(object), php_phongo_int64_ce)) {
 			php_phongo_int64_t* intern = Z_INT64_OBJ_P(object);
 
 			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Int64");
 			bson_append_int64(bson, key, key_len, intern->integer);
 			return;
 		}
-		if (instanceof_function(Z_OBJCE_P(object), php_phongo_regex_ce TSRMLS_CC)) {
+		if (instanceof_function(Z_OBJCE_P(object), php_phongo_regex_ce)) {
 			php_phongo_regex_t* intern = Z_REGEX_OBJ_P(object);
 
 			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Regex");
 			bson_append_regex(bson, key, key_len, intern->pattern, intern->flags);
 			return;
 		}
-		if (instanceof_function(Z_OBJCE_P(object), php_phongo_javascript_ce TSRMLS_CC)) {
+		if (instanceof_function(Z_OBJCE_P(object), php_phongo_javascript_ce)) {
 			php_phongo_javascript_t* intern = Z_JAVASCRIPT_OBJ_P(object);
 
 			if (intern->scope) {
@@ -256,26 +196,26 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 			}
 			return;
 		}
-		if (instanceof_function(Z_OBJCE_P(object), php_phongo_timestamp_ce TSRMLS_CC)) {
+		if (instanceof_function(Z_OBJCE_P(object), php_phongo_timestamp_ce)) {
 			php_phongo_timestamp_t* intern = Z_TIMESTAMP_OBJ_P(object);
 
 			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Timestamp");
 			bson_append_timestamp(bson, key, key_len, intern->timestamp, intern->increment);
 			return;
 		}
-		if (instanceof_function(Z_OBJCE_P(object), php_phongo_maxkey_ce TSRMLS_CC)) {
+		if (instanceof_function(Z_OBJCE_P(object), php_phongo_maxkey_ce)) {
 			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding MaxKey");
 			bson_append_maxkey(bson, key, key_len);
 			return;
 		}
-		if (instanceof_function(Z_OBJCE_P(object), php_phongo_minkey_ce TSRMLS_CC)) {
+		if (instanceof_function(Z_OBJCE_P(object), php_phongo_minkey_ce)) {
 			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding MinKey");
 			bson_append_minkey(bson, key, key_len);
 			return;
 		}
 
 		/* Deprecated types */
-		if (instanceof_function(Z_OBJCE_P(object), php_phongo_dbpointer_ce TSRMLS_CC)) {
+		if (instanceof_function(Z_OBJCE_P(object), php_phongo_dbpointer_ce)) {
 			bson_oid_t              oid;
 			php_phongo_dbpointer_t* intern = Z_DBPOINTER_OBJ_P(object);
 
@@ -284,27 +224,27 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 			bson_append_dbpointer(bson, key, key_len, intern->ref, &oid);
 			return;
 		}
-		if (instanceof_function(Z_OBJCE_P(object), php_phongo_symbol_ce TSRMLS_CC)) {
+		if (instanceof_function(Z_OBJCE_P(object), php_phongo_symbol_ce)) {
 			php_phongo_symbol_t* intern = Z_SYMBOL_OBJ_P(object);
 
 			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Symbol");
 			bson_append_symbol(bson, key, key_len, intern->symbol, intern->symbol_len);
 			return;
 		}
-		if (instanceof_function(Z_OBJCE_P(object), php_phongo_undefined_ce TSRMLS_CC)) {
+		if (instanceof_function(Z_OBJCE_P(object), php_phongo_undefined_ce)) {
 			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Undefined");
 			bson_append_undefined(bson, key, key_len);
 			return;
 		}
 
-		phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "Unexpected %s instance: %s", ZSTR_VAL(php_phongo_type_ce->name), ZSTR_VAL(Z_OBJCE_P(object)->name));
+		phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Unexpected %s instance: %s", ZSTR_VAL(php_phongo_type_ce->name), ZSTR_VAL(Z_OBJCE_P(object)->name));
 		return;
 	} else {
 		bson_t child;
 
 		mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding document");
 		bson_append_document_begin(bson, key, key_len, &child);
-		php_phongo_zval_to_bson_internal(object, field_path, flags, &child, NULL TSRMLS_CC);
+		php_phongo_zval_to_bson_internal(object, field_path, flags, &child, NULL);
 		bson_append_document_end(bson, &child);
 	}
 } /* }}} */
@@ -312,18 +252,15 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 /* Appends the zval argument to the BSON document. If the argument is an object,
  * or an array that should be serialized as an embedded document, this function
  * will defer to php_phongo_bson_append_object(). */
-static void php_phongo_bson_append(bson_t* bson, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, const char* key, long key_len, zval* entry TSRMLS_DC) /* {{{ */
+static void php_phongo_bson_append(bson_t* bson, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, const char* key, long key_len, zval* entry) /* {{{ */
 {
 	php_phongo_field_path_write_item_at_current_level(field_path, key);
 
-#if PHP_VERSION_ID >= 70000
 try_again:
-#endif
 	switch (Z_TYPE_P(entry)) {
 		case IS_NULL:
 			bson_append_null(bson, key, key_len);
 			break;
-#if PHP_VERSION_ID >= 70000
 		case IS_TRUE:
 			bson_append_bool(bson, key, key_len, true);
 			break;
@@ -331,11 +268,6 @@ try_again:
 		case IS_FALSE:
 			bson_append_bool(bson, key, key_len, false);
 			break;
-#else
-		case IS_BOOL:
-			bson_append_bool(bson, key, key_len, Z_BVAL_P(entry));
-			break;
-#endif
 
 		case IS_LONG:
 			BSON_APPEND_INT(bson, key, key_len, Z_LVAL_P(entry));
@@ -350,19 +282,19 @@ try_again:
 				bson_append_utf8(bson, key, key_len, Z_STRVAL_P(entry), Z_STRLEN_P(entry));
 			} else {
 				char* path_string = php_phongo_field_path_as_string(field_path);
-				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "Detected invalid UTF-8 for field path \"%s\": %s", path_string, Z_STRVAL_P(entry));
+				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Detected invalid UTF-8 for field path \"%s\": %s", path_string, Z_STRVAL_P(entry));
 				efree(path_string);
 			}
 			break;
 
 		case IS_ARRAY:
-			if (php_phongo_is_array_or_document(entry TSRMLS_CC) == IS_ARRAY) {
+			if (php_phongo_is_array_or_document(entry) == IS_ARRAY) {
 				bson_t     child;
 				HashTable* tmp_ht = HASH_OF(entry);
 
 				if (!php_phongo_zend_hash_apply_protection_begin(tmp_ht)) {
 					char* path_string = php_phongo_field_path_as_string(field_path);
-					phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "Detected recursion for field path \"%s\"", path_string);
+					phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Detected recursion for field path \"%s\"", path_string);
 					efree(path_string);
 					break;
 				}
@@ -370,7 +302,7 @@ try_again:
 				bson_append_array_begin(bson, key, key_len, &child);
 				php_phongo_field_path_write_type_at_current_level(field_path, PHONGO_FIELD_PATH_ITEM_ARRAY);
 				field_path->size++;
-				php_phongo_zval_to_bson_internal(entry, field_path, flags, &child, NULL TSRMLS_CC);
+				php_phongo_zval_to_bson_internal(entry, field_path, flags, &child, NULL);
 				field_path->size--;
 				bson_append_array_end(bson, &child);
 
@@ -384,47 +316,36 @@ try_again:
 
 			if (!php_phongo_zend_hash_apply_protection_begin(tmp_ht)) {
 				char* path_string = php_phongo_field_path_as_string(field_path);
-				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "Detected recursion for field path \"%s\"", path_string);
+				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Detected recursion for field path \"%s\"", path_string);
 				efree(path_string);
 				break;
 			}
 
 			php_phongo_field_path_write_type_at_current_level(field_path, PHONGO_FIELD_PATH_ITEM_DOCUMENT);
 			field_path->size++;
-			php_phongo_bson_append_object(bson, field_path, flags, key, key_len, entry TSRMLS_CC);
+			php_phongo_bson_append_object(bson, field_path, flags, key, key_len, entry);
 			field_path->size--;
 
 			php_phongo_zend_hash_apply_protection_end(tmp_ht);
 			break;
 		}
 
-#if PHP_VERSION_ID >= 70000
-		case IS_INDIRECT:
-			php_phongo_bson_append(bson, field_path, flags, key, key_len, Z_INDIRECT_P(entry) TSRMLS_DC);
-			break;
-
 		case IS_REFERENCE:
 			ZVAL_DEREF(entry);
 			goto try_again;
-#endif
 
 		default: {
 			char* path_string = php_phongo_field_path_as_string(field_path);
-			phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "Detected unsupported PHP type for field path \"%s\": %d (%s)", path_string, Z_TYPE_P(entry), zend_get_type_by_const(Z_TYPE_P(entry)));
+			phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Detected unsupported PHP type for field path \"%s\": %d (%s)", path_string, Z_TYPE_P(entry), zend_get_type_by_const(Z_TYPE_P(entry)));
 			efree(path_string);
 		}
 	}
 } /* }}} */
 
-static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, bson_t* bson, bson_t** bson_out TSRMLS_DC) /* {{{ */
+static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, bson_t* bson, bson_t** bson_out) /* {{{ */
 {
 	HashTable* ht_data = NULL;
-#if PHP_VERSION_ID >= 70000
-	zval obj_data;
-#else
-	HashPosition pos;
-	zval* obj_data = NULL;
-#endif
+	zval       obj_data;
 
 	/* If we will be encoding a class that may contain protected and private
 	 * properties, we'll need to filter them out later. */
@@ -439,12 +360,8 @@ static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path*
 
 	switch (Z_TYPE_P(data)) {
 		case IS_OBJECT:
-			if (instanceof_function(Z_OBJCE_P(data), php_phongo_serializable_ce TSRMLS_CC)) {
-#if PHP_VERSION_ID >= 70000
+			if (instanceof_function(Z_OBJCE_P(data), php_phongo_serializable_ce)) {
 				zend_call_method_with_0_params(data, NULL, NULL, BSON_SERIALIZE_FUNC_NAME, &obj_data);
-#else
-				zend_call_method_with_0_params(&data, NULL, NULL, BSON_SERIALIZE_FUNC_NAME, &obj_data);
-#endif
 
 				if (Z_ISUNDEF(obj_data)) {
 					/* zend_call_method() failed or bsonSerialize() threw an
@@ -452,38 +369,21 @@ static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path*
 					return;
 				}
 
-#if PHP_VERSION_ID >= 70000
-				if (Z_TYPE(obj_data) != IS_ARRAY && !(Z_TYPE(obj_data) == IS_OBJECT && instanceof_function(Z_OBJCE(obj_data), zend_standard_class_def TSRMLS_CC))) {
-#else
-				if (Z_TYPE_P(obj_data) != IS_ARRAY && !(Z_TYPE_P(obj_data) == IS_OBJECT && instanceof_function(Z_OBJCE_P(obj_data), zend_standard_class_def TSRMLS_CC))) {
-#endif
+				if (Z_TYPE(obj_data) != IS_ARRAY && !(Z_TYPE(obj_data) == IS_OBJECT && instanceof_function(Z_OBJCE(obj_data), zend_standard_class_def))) {
 					phongo_throw_exception(
-						PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC,
+						PHONGO_ERROR_UNEXPECTED_VALUE,
 						"Expected %s::%s() to return an array or stdClass, %s given",
 						ZSTR_VAL(Z_OBJCE_P(data)->name),
 						BSON_SERIALIZE_FUNC_NAME,
-#if PHP_VERSION_ID >= 70000
-						PHONGO_ZVAL_CLASS_OR_TYPE_NAME(obj_data)
-#else
-						PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(obj_data)
-#endif
-					);
+						PHONGO_ZVAL_CLASS_OR_TYPE_NAME(obj_data));
 
 					goto cleanup;
 				}
 
-#if PHP_VERSION_ID >= 70000
 				ht_data = HASH_OF(&obj_data);
-#else
-				ht_data = HASH_OF(obj_data);
-#endif
 
-				if (instanceof_function(Z_OBJCE_P(data), php_phongo_persistable_ce TSRMLS_CC)) {
-#if PHP_VERSION_ID >= 70000
+				if (instanceof_function(Z_OBJCE_P(data), php_phongo_persistable_ce)) {
 					bson_append_binary(bson, PHONGO_ODM_FIELD_NAME, -1, 0x80, (const uint8_t*) Z_OBJCE_P(data)->name->val, Z_OBJCE_P(data)->name->len);
-#else
-					bson_append_binary(bson, PHONGO_ODM_FIELD_NAME, -1, 0x80, (const uint8_t*) Z_OBJCE_P(data)->name, strlen(Z_OBJCE_P(data)->name));
-#endif
 					/* Ensure that we ignore an existing key with the same name
 					 * if one exists in the bsonSerialize() return value. */
 					skip_odm_field = true;
@@ -492,12 +392,12 @@ static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path*
 				break;
 			}
 
-			if (instanceof_function(Z_OBJCE_P(data), php_phongo_type_ce TSRMLS_CC)) {
-				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "%s instance %s cannot be serialized as a root element", ZSTR_VAL(php_phongo_type_ce->name), ZSTR_VAL(Z_OBJCE_P(data)->name));
+			if (instanceof_function(Z_OBJCE_P(data), php_phongo_type_ce)) {
+				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "%s instance %s cannot be serialized as a root element", ZSTR_VAL(php_phongo_type_ce->name), ZSTR_VAL(Z_OBJCE_P(data)->name));
 				return;
 			}
 
-			ht_data                 = Z_OBJ_HT_P(data)->get_properties(data TSRMLS_CC);
+			ht_data                 = Z_OBJ_HT_P(data)->get_properties(data);
 			ht_data_from_properties = true;
 			break;
 
@@ -509,13 +409,12 @@ static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path*
 			return;
 	}
 
-#if PHP_VERSION_ID >= 70000
 	{
 		zend_string* string_key = NULL;
 		zend_ulong   num_key    = 0;
 		zval*        value;
 
-		ZEND_HASH_FOREACH_KEY_VAL(ht_data, num_key, string_key, value)
+		ZEND_HASH_FOREACH_KEY_VAL_IND(ht_data, num_key, string_key, value)
 		{
 			if (string_key) {
 				if (ht_data_from_properties) {
@@ -526,7 +425,7 @@ static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path*
 				}
 
 				if (strlen(ZSTR_VAL(string_key)) != ZSTR_LEN(string_key)) {
-					phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "BSON keys cannot contain null bytes. Unexpected null byte after \"%s\".", ZSTR_VAL(string_key));
+					phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "BSON keys cannot contain null bytes. Unexpected null byte after \"%s\".", ZSTR_VAL(string_key));
 
 					goto cleanup;
 				}
@@ -549,68 +448,12 @@ static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path*
 				zend_string_addref(string_key);
 			}
 
-			php_phongo_bson_append(bson, field_path, flags & ~PHONGO_BSON_ADD_ID, ZSTR_VAL(string_key), strlen(ZSTR_VAL(string_key)), value TSRMLS_CC);
+			php_phongo_bson_append(bson, field_path, flags & ~PHONGO_BSON_ADD_ID, ZSTR_VAL(string_key), strlen(ZSTR_VAL(string_key)), value);
 
 			zend_string_release(string_key);
 		}
 		ZEND_HASH_FOREACH_END();
 	}
-#else
-	zend_hash_internal_pointer_reset_ex(ht_data, &pos);
-	for (;; zend_hash_move_forward_ex(ht_data, &pos)) {
-		char* string_key = NULL;
-		uint string_key_len = 0;
-		ulong num_key = 0;
-		zval** value;
-		int hash_type;
-
-		hash_type = zend_hash_get_current_key_ex(ht_data, &string_key, &string_key_len, &num_key, 0, &pos);
-
-		if (hash_type == HASH_KEY_NON_EXISTENT) {
-			break;
-		}
-
-		if (zend_hash_get_current_data_ex(ht_data, (void**) &value, &pos) == FAILURE) {
-			break;
-		}
-
-		if (hash_type == HASH_KEY_IS_STRING) {
-			if (ht_data_from_properties) {
-				/* Skip protected and private properties */
-				if (string_key[0] == '\0' && string_key_len > 1) {
-					continue;
-				}
-			}
-
-			if (strlen(string_key) != string_key_len - 1) {
-				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "BSON keys cannot contain null bytes. Unexpected null byte after \"%s\".", ZSTR_VAL(string_key));
-
-				goto cleanup;
-			}
-
-			if (skip_odm_field && !strcmp(string_key, PHONGO_ODM_FIELD_NAME)) {
-				continue;
-			}
-
-			if (flags & PHONGO_BSON_ADD_ID) {
-				if (!strcmp(string_key, "_id")) {
-					flags &= ~PHONGO_BSON_ADD_ID;
-				}
-			}
-		}
-
-		/* Ensure we're working with a string key */
-		if (hash_type == HASH_KEY_IS_LONG) {
-			spprintf(&string_key, 0, "%ld", num_key);
-		}
-
-		php_phongo_bson_append(bson, field_path, flags & ~PHONGO_BSON_ADD_ID, string_key, strlen(string_key), *value TSRMLS_CC);
-
-		if (hash_type == HASH_KEY_IS_LONG) {
-			efree(string_key);
-		}
-	}
-#endif
 
 	if (flags & PHONGO_BSON_ADD_ID) {
 		bson_oid_t oid;
@@ -628,7 +471,7 @@ static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path*
 		if (bson_iter_init_find(&iter, bson, "_id") && !bson_append_iter(*bson_out, NULL, 0, &iter)) {
 			/* This should not be able to happen since we are copying from
 			 * within a valid bson_t. */
-			phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "Error copying \"_id\" field from encoded document");
+			phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Error copying \"_id\" field from encoded document");
 
 			goto cleanup;
 		}
@@ -643,15 +486,40 @@ cleanup:
 /* Converts the array or object argument to a BSON document. If the object is an
  * instance of MongoDB\BSON\Serializable, the return value of bsonSerialize()
  * will be used. */
-void php_phongo_zval_to_bson(zval* data, php_phongo_bson_flags_t flags, bson_t* bson, bson_t** bson_out TSRMLS_DC) /* {{{ */
+void php_phongo_zval_to_bson(zval* data, php_phongo_bson_flags_t flags, bson_t* bson, bson_t** bson_out) /* {{{ */
 {
 	php_phongo_field_path* field_path = php_phongo_field_path_alloc(false);
 
-	php_phongo_zval_to_bson_internal(data, field_path, flags, bson, bson_out TSRMLS_CC);
+	php_phongo_zval_to_bson_internal(data, field_path, flags, bson, bson_out);
 
 	php_phongo_field_path_free(field_path);
 } /* }}} */
 
+/* Converts the argument to a bson_value_t. If the object is an instance of
+ * MongoDB\BSON\Serializable, the return value of bsonSerialize() will be
+ * used. */
+void php_phongo_zval_to_bson_value(zval* data, php_phongo_bson_flags_t flags, bson_value_t* value) /* {{{ */
+{
+	bson_iter_t iter;
+	bson_t      bson = BSON_INITIALIZER;
+
+	zval* data_object = ecalloc(sizeof(zval), 1);
+
+	array_init_size(data_object, 1);
+	add_assoc_zval(data_object, "data", data);
+
+	Z_TRY_ADDREF_P(data);
+
+	php_phongo_zval_to_bson(data_object, flags, &bson, NULL);
+
+	if (bson_iter_init_find(&iter, &bson, "data")) {
+		bson_value_copy(bson_iter_value(&iter), value);
+	}
+
+	zval_ptr_dtor(data_object);
+	efree(data_object);
+} /* }}} */
+
 /*
  * Local variables:
  * tab-width: 4
