@@ -14,6 +14,10 @@
  * limitations under the License.
  */
 
+#if !defined(MONGOC_INSIDE) && !defined(MONGOC_COMPILATION)
+#error "Only <mongoc/mongoc.h> can be included directly."
+#endif
+
 
 #ifndef MONGOC_CONFIG_H
 #define MONGOC_CONFIG_H
@@ -53,7 +57,7 @@
  * MONGOC_ENABLE_SSL_SECURE_TRANSPORT is set from configure to determine if we are
  * compiled with Native SSL support on Darwin
  */
-#define MONGOC_ENABLE_SSL_SECURE_TRANSPORT 0
+#define MONGOC_ENABLE_SSL_SECURE_TRANSPORT 1
 
 #if MONGOC_ENABLE_SSL_SECURE_TRANSPORT != 1
 #  undef MONGOC_ENABLE_SSL_SECURE_TRANSPORT
@@ -64,7 +68,7 @@
  * MONGOC_ENABLE_CRYPTO_COMMON_CRYPTO is set from configure to determine if we are
  * compiled with Native Crypto support on Darwin
  */
-#define MONGOC_ENABLE_CRYPTO_COMMON_CRYPTO 0
+#define MONGOC_ENABLE_CRYPTO_COMMON_CRYPTO 1
 
 #if MONGOC_ENABLE_CRYPTO_COMMON_CRYPTO != 1
 #  undef MONGOC_ENABLE_CRYPTO_COMMON_CRYPTO
@@ -86,7 +90,7 @@
  * MONGOC_ENABLE_SSL_OPENSSL is set from configure to determine if we are
  * compiled with OpenSSL support.
  */
-#define MONGOC_ENABLE_SSL_OPENSSL 1
+#define MONGOC_ENABLE_SSL_OPENSSL 0
 
 #if MONGOC_ENABLE_SSL_OPENSSL != 1
 #  undef MONGOC_ENABLE_SSL_OPENSSL
@@ -97,7 +101,7 @@
  * MONGOC_ENABLE_CRYPTO_LIBCRYPTO is set from configure to determine if we are
  * compiled with OpenSSL support.
  */
-#define MONGOC_ENABLE_CRYPTO_LIBCRYPTO 1
+#define MONGOC_ENABLE_CRYPTO_LIBCRYPTO 0
 
 #if MONGOC_ENABLE_CRYPTO_LIBCRYPTO != 1
 #  undef MONGOC_ENABLE_CRYPTO_LIBCRYPTO
@@ -139,7 +143,7 @@
 /*
  * Use ASN1_STRING_get0_data () rather than the deprecated ASN1_STRING_data
  */
-#define MONGOC_HAVE_ASN1_STRING_GET0_DATA 1
+#define MONGOC_HAVE_ASN1_STRING_GET0_DATA 0
 
 #if MONGOC_HAVE_ASN1_STRING_GET0_DATA != 1
 #  undef MONGOC_HAVE_ASN1_STRING_GET0_DATA
@@ -178,18 +182,6 @@
 #  undef MONGOC_ENABLE_SASL_SSPI
 #endif
 
-
-/*
- * MONGOC_ENABLE_SASL_GSSAPI is set from configure to determine if we are
- * compiled with GSSAPI support.
- */
-#define MONGOC_ENABLE_SASL_GSSAPI 0
-
-#if MONGOC_ENABLE_SASL_GSSAPI != 1
-#  undef MONGOC_ENABLE_SASL_GSSAPI
-#endif
-
-
 /*
  * MONGOC_HAVE_SASL_CLIENT_DONE is set from configure to determine if we
  * have SASL and its version is new enough to use sasl_client_done (),
@@ -249,7 +241,7 @@
  * MONGOC_HAVE_RES_NDESTROY is set from configure to determine if we
  * have BSD / Darwin's res_ndestroy().
  */
-#define MONGOC_HAVE_RES_NDESTROY 0
+#define MONGOC_HAVE_RES_NDESTROY 1
 
 #if MONGOC_HAVE_RES_NDESTROY != 1
 #  undef MONGOC_HAVE_RES_NDESTROY
@@ -260,7 +252,7 @@
  * MONGOC_HAVE_RES_NCLOSE is set from configure to determine if we
  * have Linux's res_nclose().
  */
-#define MONGOC_HAVE_RES_NCLOSE 1
+#define MONGOC_HAVE_RES_NCLOSE 0
 
 #if MONGOC_HAVE_RES_NCLOSE != 1
 #  undef MONGOC_HAVE_RES_NCLOSE
@@ -300,7 +292,7 @@
  * Set if we have snappy compression support
  *
  */
-#define MONGOC_ENABLE_COMPRESSION_SNAPPY 1
+#define MONGOC_ENABLE_COMPRESSION_SNAPPY 0
 
 #if MONGOC_ENABLE_COMPRESSION_SNAPPY != 1
 #  undef MONGOC_ENABLE_COMPRESSION_SNAPPY
@@ -317,6 +309,16 @@
 #  undef MONGOC_ENABLE_COMPRESSION_ZLIB
 #endif
 
+/*
+ * Set if we have zstd compression support
+ *
+ */
+#define MONGOC_ENABLE_COMPRESSION_ZSTD 0
+
+#if MONGOC_ENABLE_COMPRESSION_ZSTD != 1
+#  undef MONGOC_ENABLE_COMPRESSION_ZSTD
+#endif
+
 /*
  * Set if performance counters are available and not disabled.
  *
@@ -342,7 +344,7 @@
  * Set if we have the sched_getcpu() function for use with counters
  *
  */
-#define MONGOC_HAVE_SCHED_GETCPU 1
+#define MONGOC_HAVE_SCHED_GETCPU 0
 
 #if MONGOC_HAVE_SCHED_GETCPU != 1
 #  undef MONGOC_HAVE_SCHED_GETCPU
@@ -362,19 +364,49 @@
 /*
  * Set if we have ICU support.
  */
-#define MONGOC_ENABLE_ICU 1
+#define MONGOC_ENABLE_ICU 0
 
 #if MONGOC_ENABLE_ICU != 1
 #  undef MONGOC_ENABLE_ICU
 #endif
 
 
+/*
+ * Set if we have Client Side Encryption support.
+ */
+
+#define MONGOC_ENABLE_CLIENT_SIDE_ENCRYPTION 1
+
+#if MONGOC_ENABLE_CLIENT_SIDE_ENCRYPTION != 1
+#  undef MONGOC_ENABLE_CLIENT_SIDE_ENCRYPTION
+#endif
+
+
+/*
+ * Set if struct sockaddr_storage has __ss_family (instead of ss_family)
+ */
+
+#define MONGOC_HAVE_SS_FAMILY 0
+
+#if MONGOC_HAVE_SS_FAMILY != 1
+#  undef MONGOC_HAVE_SS_FAMILY
+#endif
+
+/*
+ * Set if building with AWS IAM support.
+ */
+#define MONGOC_ENABLE_MONGODB_AWS_AUTH 1
+
+#if MONGOC_ENABLE_MONGODB_AWS_AUTH != 1
+#  undef MONGOC_ENABLE_MONGODB_AWS_AUTH
+#endif
+
 /*
  * NOTICE:
  * If you're about to update this file and add a config flag, make sure to
  * update:
  * o The bitfield in mongoc-handshake-private.h
- * o _get_config_bitfield() in mongoc-handshake.c
+ * o _mongoc_handshake_get_config_hex_string() in mongoc-handshake.c
  * o examples/parse_handshake_cfg.py
  * o test_handshake_config_string in test-mongoc-handshake.c
  */
